searchData={"items":[{"type":"module","title":"Expression","doc":"Documentation for `Expression`, a library to parse and evaluate\n[Floip](https://floip.gitbook.io/flow-specification/expressions) compatible expressions\n\nExpression is an expression language which consists of the functions provided\nby Excel with a few additions.\n\nFunction and variable names are not case-sensitive so UPPER is equivalent to upper:\n\n```\ncontact.name -> Marshawn Lynch\nFIRST_WORD(contact.name) -> Marshawn\nfirst_word(CONTACT.NAME) -> Marshawn\n```\n\nFor templating, RapidPro uses the @ character to denote either a single variable substitution\nor the beginning of an Expression block. `@` was chosen as it is known how to type by a broad\nnumber of users regardless of keyboard. It does have the disadvantage of being used in\nemail addresses and Twitter handles, but these are rarely ambiguous and escaping can be\ndone easily via doubling of the character (`@@`).\n\nFunctions are called by using the block syntax:\n```\n10 plus 4 is @(SUM(10, 4))\n```\n\nWithin a block, `@` is not required to refer to variable in the context:\n```\nHello @(contact.name)\n```\n\nA template can contain more than one substitution or block:\n```\nHello @contact.name, you were born in @(YEAR(contact.birthday))\n```","ref":"Expression.html"},{"type":"function","title":"Expression.escape/1","doc":"","ref":"Expression.html#escape/1"},{"type":"function","title":"Expression.evaluate/3","doc":"","ref":"Expression.html#evaluate/3"},{"type":"function","title":"Expression.evaluate!/3","doc":"","ref":"Expression.html#evaluate!/3"},{"type":"function","title":"Expression.evaluate_as_boolean!/3","doc":"","ref":"Expression.html#evaluate_as_boolean!/3"},{"type":"function","title":"Expression.evaluate_as_string!/3","doc":"","ref":"Expression.html#evaluate_as_string!/3"},{"type":"function","title":"Expression.evaluate_block/3","doc":"","ref":"Expression.html#evaluate_block/3"},{"type":"function","title":"Expression.evaluate_block!/3","doc":"","ref":"Expression.html#evaluate_block!/3"},{"type":"function","title":"Expression.parse!/1","doc":"","ref":"Expression.html#parse!/1"},{"type":"function","title":"Expression.parse_expression/1","doc":"","ref":"Expression.html#parse_expression/1"},{"type":"function","title":"Expression.parse_expression!/1","doc":"","ref":"Expression.html#parse_expression!/1"},{"type":"function","title":"Expression.prewalk/2","doc":"","ref":"Expression.html#prewalk/2"},{"type":"function","title":"Expression.stringify/1","doc":"Convert an Expression type into a string.\n\nThis function is applied to all values when `Expression.evaluate_as_string!/3` is called.","ref":"Expression.html#stringify/1"},{"type":"function","title":"Expression.traverse/4","doc":"","ref":"Expression.html#traverse/4"},{"type":"type","title":"Expression.expression_type/0","doc":"","ref":"Expression.html#t:expression_type/0"},{"type":"module","title":"Expression.Autodoc","doc":"Extract `@expression_doc` attributes from modules defining callbacks\nand automatically write doctests for those.\n\nAlso inserts an `expression_docs()` function which returns a list of\nall functions and their defined expression docs.\n\nThe format is:\n\n```elixir\n@expression_doc doc: \"Construct a date from year, month, and day integers\",\n                expression: \"@date(year, month, day)\",\n                context: %{\"year\" => 2022, \"month\" => 1, \"day\" => 31},\n                result: \"2022-01-31T00:00:00Z\"\n```\n\nWhere:\n\n* `doc` is the explanatory text added to the doctest.\n* `expression` is the expression we want to test\n* `fake_expression` can optionally be the expression we want to display but not test\n* `context` is the context the expression is tested against\n* `result` is the result we're expecting to get and are asserting against\n* `fake_result` can be optionally supplied when the returning result varies\n   depending on factors we do not control, like for `now()` for example.\n   When this is used, the ExDoc tests are faked and won't actually test\n   anything so use sparingly.","ref":"Expression.Autodoc.html"},{"type":"function","title":"Expression.Autodoc.annotate_method/3","doc":"","ref":"Expression.Autodoc.html#annotate_method/3"},{"type":"function","title":"Expression.Autodoc.format_context/1","doc":"","ref":"Expression.Autodoc.html#format_context/1"},{"type":"function","title":"Expression.Autodoc.format_docs/1","doc":"","ref":"Expression.Autodoc.html#format_docs/1"},{"type":"function","title":"Expression.Autodoc.format_function_args/1","doc":"","ref":"Expression.Autodoc.html#format_function_args/1"},{"type":"function","title":"Expression.Autodoc.format_function_name/1","doc":"","ref":"Expression.Autodoc.html#format_function_name/1"},{"type":"function","title":"Expression.Autodoc.format_result/1","doc":"","ref":"Expression.Autodoc.html#format_result/1"},{"type":"function","title":"Expression.Autodoc.generate_assert/2","doc":"","ref":"Expression.Autodoc.html#generate_assert/2"},{"type":"function","title":"Expression.Autodoc.generate_ex_doc/5","doc":"","ref":"Expression.Autodoc.html#generate_ex_doc/5"},{"type":"function","title":"Expression.Autodoc.get_existing_docstring/1","doc":"","ref":"Expression.Autodoc.html#get_existing_docstring/1"},{"type":"function","title":"Expression.Autodoc.get_expression/1","doc":"","ref":"Expression.Autodoc.html#get_expression/1"},{"type":"function","title":"Expression.Autodoc.stringify/1","doc":"","ref":"Expression.Autodoc.html#stringify/1"},{"type":"function","title":"Expression.Autodoc.type_of/1","doc":"","ref":"Expression.Autodoc.html#type_of/1"},{"type":"function","title":"Expression.Autodoc.update_annotations/4","doc":"","ref":"Expression.Autodoc.html#update_annotations/4"},{"type":"module","title":"Expression.Callbacks","doc":"Use this module to implement one's own callbacks.\nThe standard callbacks available are implemented in `Expression.Callbacks.Standard`.\n\n```elixir\ndefmodule MyCallbacks do\n  use Expression.Callbacks\n\n  @doc \"\"\"\n  Roll a dice and randomly return a number between 1 and 6.\n  \"\"\"\n  def dice_roll(ctx) do\n    Enum.random(1..6)\n  end\n\nend\n```","ref":"Expression.Callbacks.html"},{"type":"function","title":"Expression.Callbacks.atom_function_name/1","doc":"Convert a string function name into an atom meant to handle\nthat function\n\nReserved words such as `and`, `if`, and `or` are automatically suffixed\nwith an `_` underscore.","ref":"Expression.Callbacks.html#atom_function_name/1"},{"type":"function","title":"Expression.Callbacks.handle/4","doc":"Handle a function call while evaluating the AST.\n\nHandlers in this module are either:\n\n1. The function name as is\n2. The function name with an underscore suffix if the function name is a reserved word\n3. The function name suffixed with `_vargs` if the takes a variable set of arguments","ref":"Expression.Callbacks.html#handle/4"},{"type":"function","title":"Expression.Callbacks.implements/3","doc":"","ref":"Expression.Callbacks.html#implements/3"},{"type":"module","title":"Expression.Callbacks.Standard","doc":"The function callbacks for the standard function set available\nin FLOIP expressions.\n\nThis should be relatively swappable with another implementation.\nThe only requirement is the `handle/3` function.\n\nFLOIP functions are case insensitive. All functions in this callback\nmodule are implemented as lowercase names.\n\nSome functions accept a variable amount of arguments. Elixir doesn't\nsupport variable arguments in functions.\n\nIf a function accepts a variable number of arguments the convention\nis to call the ` _vargs/2` callback where the context\nis given as the first argument and the argument list as a second\nargument.\n\nReserved names such as `and`, `if`, and `or` are suffixed with an\nunderscore.","ref":"Expression.Callbacks.Standard.html"},{"type":"function","title":"Expression.Callbacks.Standard.abs/2","doc":"Returns the absolute value of a number","ref":"Expression.Callbacks.Standard.html#abs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.abs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> abs(-1)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @abs(-1) ...\"\n\"1\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"abs(-1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 1 = result\n    1\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@abs(-1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#abs/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.and_vargs/2","doc":"Returns `true` if and only if all its arguments evaluate to `true`","ref":"Expression.Callbacks.Standard.html#and_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.and_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}}\n```\n\n\n```\n> contact.gender = \"F\" and contact.age >= 18\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @and(contact.gender = \"F\", contact.age >= 18) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   %{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   %{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#and_vargs/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.and_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}}\n```\n\n\n```\n> contact.gender = \"F\" and contact.age >= 18\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @and(contact.gender = \"F\", contact.age >= 18) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   %{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   %{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#and_vargs/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.append/3","doc":"Appends an item or a list of items to a given list.","ref":"Expression.Callbacks.Standard.html#append/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.append/3","doc":"When used in the following Stack expression it returns a  value of type **List with values String, String, String**: `[\n  \"A\",\n  \"B\",\n  \"C\"\n]`.\n\n```\n> append([\"A\", \"B\"], \"C\")\n[\"A\", \"B\", \"C\"]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @append([\"A\", \"B\"], \"C\") ...\"\n\"ABC\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"append([\\\"A\\\", \\\"B\\\"], \\\"C\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert [\"A\", \"B\", \"C\"] = result\n    [\"A\", \"B\", \"C\"]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@append([\\\"A\\\", \\\"B\\\"], \\\"C\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"ABC\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#append/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.append/3","doc":"When used in the following Stack expression it returns a  value of type **List with values String, String, String, String**: `[\n  \"A\",\n  \"B\",\n  \"C\",\n  \"B\"\n]`.\n\n```\n> append([\"A\", \"B\"], [\"C\", \"B\"])\n[\"A\", \"B\", \"C\", \"B\"]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @append([\"A\", \"B\"], [\"C\", \"B\"]) ...\"\n\"ABCB\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"append([\\\"A\\\", \\\"B\\\"], [\\\"C\\\", \\\"B\\\"])\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert [\"A\", \"B\", \"C\", \"B\"] = result\n    [\"A\", \"B\", \"C\", \"B\"]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@append([\\\"A\\\", \\\"B\\\"], [\\\"C\\\", \\\"B\\\"])\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"ABCB\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#append/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.char/2","doc":"Returns the character specified by a number\n\n```\n> \"As easy as @char(65), @char(66), @char(67)\"\n\"As easy as A, B, C\"\n```","ref":"Expression.Callbacks.Standard.html#char/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.char/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"A\"`.\n\n```\n> char(65)\n\"A\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @char(65) ...\"\n\"A\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"char(65)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"A\" = result\n    \"A\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@char(65)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"A\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#char/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.clean/2","doc":"Removes all non-printable characters from a text string","ref":"Expression.Callbacks.Standard.html#clean/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.clean/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ABC\"` when used with the following context:\n\n```elixir\n%{\"value\" => <<65, 0, 66, 0, 67>>}\n```\n\n\n```\n> clean(value)\n\"ABC\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @clean(value) ...\"\n\"ABC\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"clean(value)\",\n    ...>   %{\"value\" => <<65, 0, 66, 0, 67>>},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"ABC\" = result\n    \"ABC\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@clean(value)\",\n    ...>   %{\"value\" => <<65, 0, 66, 0, 67>>},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"ABC\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#clean/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.code/2","doc":"Returns a numeric code for the first character in a text string\n\n```\n> \"The numeric code of A is @CODE(\\\"A\\\")\"\n\"The numeric code of A is 65\"\n```","ref":"Expression.Callbacks.Standard.html#code/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.code/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `65`.\n\n```\n> code(\"A\")\n65\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @code(\"A\") ...\"\n\"65\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"code(\\\"A\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 65 = result\n    65\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@code(\\\"A\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"65\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#code/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.concatenate_vargs/2","doc":"Joins text strings into one text string\n\n```\n> \"Your name is @CONCATENATE(contact.first_name, \\\" \\\", contact.last_name)\"\n\"Your name is name surname\"\n```","ref":"Expression.Callbacks.Standard.html#concatenate_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.concatenate_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"name surname\"` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}}\n```\n\n\n```\n> concatenate(contact.first_name, \" \", contact.last_name)\n\"name surname\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @concatenate(contact.first_name, \" \", contact.last_name) ...\"\n\"name surname\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"concatenate(contact.first_name, \\\" \\\", contact.last_name)\",\n    ...>   %{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"name surname\" = result\n    \"name surname\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@concatenate(contact.first_name, \\\" \\\", contact.last_name)\",\n    ...>   %{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"name surname\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#concatenate_vargs/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.date/4","doc":"Defines a new date value","ref":"Expression.Callbacks.Standard.html#date/4"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.date/4","doc":"> Construct a date from year, month, and day integers\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `\"2022-01-31\"` when used with the following context:\n\n```elixir\n%{\"day\" => 31, \"month\" => 1, \"year\" => 2022}\n```\n\n\n```\n> date(year, month, day)\n~D[2022-01-31]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @date(year, month, day) ...\"\n\"2022-01-31\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"date(year, month, day)\",\n    ...>   %{\"day\" => 31, \"month\" => 1, \"year\" => 2022},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~D[2022-01-31] = result\n    ~D[2022-01-31]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@date(year, month, day)\",\n    ...>   %{\"day\" => 31, \"month\" => 1, \"year\" => 2022},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-01-31\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#date/4-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.datetime_add/4","doc":"Calculates a new datetime based on the offset and unit provided.\n\nThe unit can be any of the following values:\n\n* \"Y\" for years\n* \"M\" for months\n* \"W\" for weeks\n* \"D\" for days\n* \"h\" for hours\n* \"m\" for minutes\n* \"s\" for seconds\n\nSpecifying a negative offset results in date calculations back in time.","ref":"Expression.Callbacks.Standard.html#datetime_add/4"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.datetime_add/4","doc":"> Calculates a new datetime based on the offset and unit provided.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2022-08-31T00:00:00Z\"` when used with the following context:\n\n```elixir\n%{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => \"1\", \"unit\" => \"M\"}\n```\n\n\n```\n> datetime_add(datetime, offset, unit)\n~U[2022-08-31 00:00:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(datetime, offset, unit) ...\"\n\"2022-08-31T00:00:00Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datetime_add(datetime, offset, unit)\",\n    ...>   %{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => \"1\", \"unit\" => \"M\"},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2022-08-31 00:00:00Z] = result\n    ~U[2022-08-31 00:00:00Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datetime_add(datetime, offset, unit)\",\n    ...>   %{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => \"1\", \"unit\" => \"M\"},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-08-31T00:00:00Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datetime_add/4-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.datetime_add/4","doc":"> Leap year handling in a leap year.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2020-02-29T00:00:00.000000Z\"`.\n\n```\n> datetime_add(date(2020, 02, 28), 1, \"D\")\n~U[2020-02-29 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2020, 02, 28), 1, \"D\") ...\"\n\"2020-02-29T00:00:00.000000Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datetime_add(date(2020, 02, 28), 1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2020-02-29 00:00:00.000000Z] = result\n    ~U[2020-02-29 00:00:00.000000Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datetime_add(date(2020, 02, 28), 1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2020-02-29T00:00:00.000000Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datetime_add/4-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.datetime_add/4","doc":"> Leap year handling outside of a leap year.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2021-03-01T00:00:00.000000Z\"`.\n\n```\n> datetime_add(date(2021, 02, 28), 1, \"D\")\n~U[2021-03-01 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2021, 02, 28), 1, \"D\") ...\"\n\"2021-03-01T00:00:00.000000Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datetime_add(date(2021, 02, 28), 1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2021-03-01 00:00:00.000000Z] = result\n    ~U[2021-03-01 00:00:00.000000Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datetime_add(date(2021, 02, 28), 1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2021-03-01T00:00:00.000000Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datetime_add/4-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.datetime_add/4","doc":"> Negative offsets\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2020-02-28T00:00:00.000000Z\"`.\n\n```\n> datetime_add(date(2020, 02, 29), -1, \"D\")\n~U[2020-02-28 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2020, 02, 29), -1, \"D\") ...\"\n\"2020-02-28T00:00:00.000000Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datetime_add(date(2020, 02, 29), -1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2020-02-28 00:00:00.000000Z] = result\n    ~U[2020-02-28 00:00:00.000000Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datetime_add(date(2020, 02, 29), -1, \\\"D\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2020-02-28T00:00:00.000000Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datetime_add/4-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.datevalue/2","doc":"","ref":"Expression.Callbacks.Standard.html#datevalue/2"},{"type":"function","title":"Expression.Callbacks.Standard.datevalue/3","doc":"Converts date stored in text to an actual date object and\nformats it using `strftime` formatting.\n\nIt will fallback to \"%Y-%m-%d %H:%M:%S\" if no formatting is supplied","ref":"Expression.Callbacks.Standard.html#datevalue/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.datevalue/3","doc":"> Convert a date from a piece of text to a formatted date string\n\n\nWhen used in the following Stack expression it returns a complex **String** type of default value:\n```\n\"2022-01-01 00:00:00\"\n```\nwith the following fields:\n\n* *date* of type **Date**\n* *datetime* of type **DateTime**\n.\n\n```\n> datevalue(\"2022-01-01\")\n%{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01], \"datetime\" => ~U[2022-01-01 00:00:00Z]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(\"2022-01-01\") ...\"\n\"2022-01-01 00:00:00\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datevalue(\\\"2022-01-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01], \"datetime\" => ~U[2022-01-01 00:00:00Z]} = result\n    %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01], \"datetime\" => ~U[2022-01-01 00:00:00Z]}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datevalue(\\\"2022-01-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-01-01 00:00:00\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datevalue/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.datevalue/3","doc":"> Convert a date from a piece of text and read the date field\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `\"2022-01-01\"`.\n\n```\n> datevalue(\"2022-01-01\").date\n~D[2022-01-01]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(\"2022-01-01\").date ...\"\n\"2022-01-01\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datevalue(\\\"2022-01-01\\\").date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~D[2022-01-01] = result\n    ~D[2022-01-01]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datevalue(\\\"2022-01-01\\\").date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-01-01\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datevalue/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.datevalue/3","doc":"> Convert a date value and read the date field\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `\"2022-01-01\"`.\n\n```\n> datevalue(date(2022, 1, 1)).date\n~D[2022-01-01]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(date(2022, 1, 1)).date ...\"\n\"2022-01-01\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"datevalue(date(2022, 1, 1)).date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~D[2022-01-01] = result\n    ~D[2022-01-01]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@datevalue(date(2022, 1, 1)).date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-01-01\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#datevalue/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.day/2","doc":"Returns only the day of the month of a date (1 to 31)","ref":"Expression.Callbacks.Standard.html#day/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.day/2","doc":"> Getting today's day of the month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `10`.\n\n```\n> day(date(2022, 9, 10))\n10\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @day(date(2022, 9, 10)) ...\"\n\"10\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"day(date(2022, 9, 10))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 10 = result\n    10\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@day(date(2022, 9, 10))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"10\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#day/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.day/2","doc":"> Getting today's day of the month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `16`.\n\n```\n> day(now())\n16\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @day(now()) ...\"\n\"16\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"day(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 16 = result\n    16\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@day(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"16\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#day/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.delete/3","doc":"Deletes an element from a map by the given key.","ref":"Expression.Callbacks.Standard.html#delete/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.delete/3","doc":"When used in the following Stack expression it returns a  value of type **Map**: `{\n  \"age\": 32\n}` when used with the following context:\n\n```elixir\n%{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}}\n```\n\n\n```\n> delete(patient, \"gender\")\n%{\"age\" => 32}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @delete(patient, \"gender\") ...\"\n\"%{\"age\" => 32}\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"delete(patient, \\\"gender\\\")\",\n    ...>   %{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"age\" => 32} = result\n    %{\"age\" => 32}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@delete(patient, \\\"gender\\\")\",\n    ...>   %{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"%{\\\"age\\\" => 32}\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#delete/3-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.edate/3","doc":"Moves a date by the given number of months","ref":"Expression.Callbacks.Standard.html#edate/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.edate/3","doc":"> Move the date in a date object by 1 month\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2022-02-01T00:00:00Z\"` when used with the following context:\n\n```elixir\n%{right_now: ~U[2022-01-01 00:00:00Z]}\n```\n\n\n```\n> edate(right_now, 1)\n~U[2022-02-01 00:00:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @edate(right_now, 1) ...\"\n\"2022-02-01T00:00:00Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"edate(right_now, 1)\",\n    ...>   %{right_now: ~U[2022-01-01 00:00:00Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2022-02-01 00:00:00Z] = result\n    ~U[2022-02-01 00:00:00Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@edate(right_now, 1)\",\n    ...>   %{right_now: ~U[2022-01-01 00:00:00Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-02-01T00:00:00Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#edate/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.edate/3","doc":"> Move the date store in a piece of text by 1 month\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `\"2022-11-10\"`.\n\n```\n> edate(\"2022-10-10\", 1)\n~D[2022-11-10]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @edate(\"2022-10-10\", 1) ...\"\n\"2022-11-10\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"edate(\\\"2022-10-10\\\", 1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~D[2022-11-10] = result\n    ~D[2022-11-10]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@edate(\\\"2022-10-10\\\", 1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-11-10\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#edate/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.expression_docs/0","doc":"Return a list of all functions annotated with @expression_docs","ref":"Expression.Callbacks.Standard.html#expression_docs/0"},{"type":"function","title":"Expression.Callbacks.Standard.first_word/2","doc":"Returns the first word in the given text - equivalent to WORD(text, 1)","ref":"Expression.Callbacks.Standard.html#first_word/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foo\"`.\n\n```\n> first_word(\"foo bar baz\")\n\"foo\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @first_word(\"foo bar baz\") ...\"\n\"foo\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"first_word(\\\"foo bar baz\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"foo\" = result\n    \"foo\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@first_word(\\\"foo bar baz\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"foo\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#first_word/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.fixed/3","doc":"Formats the given number in decimal format using a period and commas\n\n```\n> You have @fixed(contact.balance, 2) in your account\n\"You have 4.21 in your account\"\n```","ref":"Expression.Callbacks.Standard.html#fixed/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.fixed/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"4.21\"`.\n\n```\n> fixed(4.209922, 2, false)\n\"4.21\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(4.209922, 2, false) ...\"\n\"4.21\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"fixed(4.209922, 2, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"4.21\" = result\n    \"4.21\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@fixed(4.209922, 2, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"4.21\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#fixed/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.fixed/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"4,000.4242\"`.\n\n```\n> fixed(4000.424242, 4, true)\n\"4,000.4242\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(4000.424242, 4, true) ...\"\n\"4,000.4242\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"fixed(4000.424242, 4, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"4,000.4242\" = result\n    \"4,000.4242\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@fixed(4000.424242, 4, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"4,000.4242\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#fixed/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.fixed/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"3.80\"`.\n\n```\n> fixed(3.7979, 2, false)\n\"3.80\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(3.7979, 2, false) ...\"\n\"3.80\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"fixed(3.7979, 2, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"3.80\" = result\n    \"3.80\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@fixed(3.7979, 2, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3.80\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#fixed/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.fixed/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"3.80\"`.\n\n```\n> fixed(3.7979, 2)\n\"3.80\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(3.7979, 2) ...\"\n\"3.80\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"fixed(3.7979, 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"3.80\" = result\n    \"3.80\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@fixed(3.7979, 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3.80\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#fixed/3-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.fixed/4","doc":"","ref":"Expression.Callbacks.Standard.html#fixed/4"},{"type":"function","title":"Expression.Callbacks.Standard.handle/4","doc":"","ref":"Expression.Callbacks.Standard.html#handle/4"},{"type":"function","title":"Expression.Callbacks.Standard.has_all_words/3","doc":"Tests whether all the words are contained in text\n\nThe words can be in any order and may appear more than once.","ref":"Expression.Callbacks.Standard.html#has_all_words/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_all_words/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_all_words(\"the quick brown FOX\", \"the fox\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_all_words(\"the quick brown FOX\", \"the fox\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_all_words(\\\"the quick brown FOX\\\", \\\"the fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_all_words(\\\"the quick brown FOX\\\", \\\"the fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_all_words/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_all_words/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_all_words(\"the quick brown FOX\", \"red fox\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_all_words(\"the quick brown FOX\", \"red fox\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_all_words(\\\"the quick brown FOX\\\", \\\"red fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_all_words(\\\"the quick brown FOX\\\", \\\"red fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_all_words/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_any_word/3","doc":"Tests whether any of the words are contained in the text\n\nOnly one of the words needs to match and it may appear more than once.","ref":"Expression.Callbacks.Standard.html#has_any_word/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_any_word/3","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *match* of type **String**\n.\n\n```\n> has_any_word(\"The Quick Brown Fox\", \"fox quick\")\n%{\"__value__\" => true, \"match\" => \"Quick Fox\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_any_word(\"The Quick Brown Fox\", \"fox quick\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_any_word(\\\"The Quick Brown Fox\\\", \\\"fox quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"match\" => \"Quick Fox\"} = result\n    %{\"__value__\" => true, \"match\" => \"Quick Fox\"}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_any_word(\\\"The Quick Brown Fox\\\", \\\"fox quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_any_word/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_any_word/3","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\nfalse\n```\nwith the following fields:\n\n* *match* of type **Null**\n.\n\n```\n> has_any_word(\"The Quick Brown Fox\", \"yellow\")\n%{\"__value__\" => false, \"match\" => nil}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_any_word(\"The Quick Brown Fox\", \"yellow\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_any_word(\\\"The Quick Brown Fox\\\", \\\"yellow\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => false, \"match\" => nil} = result\n    %{\"__value__\" => false, \"match\" => nil}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_any_word(\\\"The Quick Brown Fox\\\", \\\"yellow\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_any_word/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_beginning/3","doc":"Tests whether text starts with beginning\n\nBoth text values are trimmed of surrounding whitespace, but otherwise matching is\nstrict without any tokenization.","ref":"Expression.Callbacks.Standard.html#has_beginning/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_beginning(\"The Quick Brown\", \"the quick\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"the quick\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"the quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"the quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_beginning/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_beginning(\"The Quick Brown\", \"the    quick\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"the    quick\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"the    quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"the    quick\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_beginning/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_beginning(\"The Quick Brown\", \"quick brown\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"quick brown\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_beginning/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.has_date/2","doc":"Tests whether `expression` contains a date formatted according to our environment\n\nThis is very naively implemented with a regular expression.","ref":"Expression.Callbacks.Standard.html#has_date/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *date* of type **Date**\n* *datetime* of type **DateTime**\n.\n\n```\n> has_date(\"the date is 15/01/2017 05:50\")\n%{\"__value__\" => true, \"date\" => ~D[2017-01-15], \"datetime\" => ~U[2017-01-15 05:50:00Z]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"the date is 15/01/2017 05:50\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date(\\\"the date is 15/01/2017 05:50\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"date\" => ~D[2017-01-15], \"datetime\" => ~U[2017-01-15 05:50:00Z]} = result\n    %{\"__value__\" => true, \"date\" => ~D[2017-01-15], \"datetime\" => ~U[2017-01-15 05:50:00Z]}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date(\\\"the date is 15/01/2017 05:50\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a  value of type **Date**: `\"2017-01-15\"`.\n\n```\n> has_date(\"the date is 15/01/2017\").date\n~D[2017-01-15]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"the date is 15/01/2017\").date ...\"\n\"2017-01-15\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date(\\\"the date is 15/01/2017\\\").date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~D[2017-01-15] = result\n    ~D[2017-01-15]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date(\\\"the date is 15/01/2017\\\").date\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2017-01-15\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a  value of type **DateTime**: `\"2017-01-15T05:50:00Z\"`.\n\n```\n> has_date(\"the date is 15/01/2017 05:50\").datetime\n~U[2017-01-15 05:50:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"the date is 15/01/2017 05:50\").datetime ...\"\n\"2017-01-15T05:50:00Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date(\\\"the date is 15/01/2017 05:50\\\").datetime\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2017-01-15 05:50:00Z] = result\n    ~U[2017-01-15 05:50:00Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date(\\\"the date is 15/01/2017 05:50\\\").datetime\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2017-01-15T05:50:00Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\nfalse\n```\nwith the following fields:\n\n* *date* of type **Null**\n* *datetime* of type **Null**\n.\n\n```\n> has_date(\"there is no date here, just a year 2017\")\n%{\"__value__\" => false, \"date\" => nil, \"datetime\" => nil}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"there is no date here, just a year 2017\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date(\\\"there is no date here, just a year 2017\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => false, \"date\" => nil, \"datetime\" => nil} = result\n    %{\"__value__\" => false, \"date\" => nil, \"datetime\" => nil}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date(\\\"there is no date here, just a year 2017\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.has_date_eq/3","doc":"Tests whether `expression` is a date equal to `date_string`","ref":"Expression.Callbacks.Standard.html#has_date_eq/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_date_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_eq(\"the date is 15/01/2017\", \"2017-01-15\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_eq(\"the date is 15/01/2017\", \"2017-01-15\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_eq(\\\"the date is 15/01/2017\\\", \\\"2017-01-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_eq(\\\"the date is 15/01/2017\\\", \\\"2017-01-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_eq/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_date_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_eq(\"there is no date here, just a year 2017\", \"2017-01-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_eq(\"there is no date here, just a year 2017\", \"2017-01-15\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_eq(\\\"there is no date here, just a year 2017\\\", \\\"2017-01-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_eq(\\\"there is no date here, just a year 2017\\\", \\\"2017-01-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_eq/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_date_gt/3","doc":"Tests whether `expression` is a date after the date `date_string`","ref":"Expression.Callbacks.Standard.html#has_date_gt/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_date_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_gt(\"the date is 15/01/2017\", \"2017-01-01\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_gt(\"the date is 15/01/2017\", \"2017-01-01\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-01-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-01-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_gt/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_date_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_gt(\"the date is 15/01/2017\", \"2017-03-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_gt(\"the date is 15/01/2017\", \"2017-03-15\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-03-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-03-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_gt/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_date_lt/3","doc":"Tests whether `expression` contains a date before the date `date_string`","ref":"Expression.Callbacks.Standard.html#has_date_lt/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_date_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_lt(\"the date is 15/01/2017\", \"2017-06-01\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_lt(\"the date is 15/01/2017\", \"2017-06-01\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_lt(\\\"the date is 15/01/2017\\\", \\\"2017-06-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_lt(\\\"the date is 15/01/2017\\\", \\\"2017-06-01\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_lt/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_date_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_lt(\"the date is 15/01/2021\", \"2017-03-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_lt(\"the date is 15/01/2021\", \"2017-03-15\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_date_lt(\\\"the date is 15/01/2021\\\", \\\"2017-03-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_date_lt(\\\"the date is 15/01/2021\\\", \\\"2017-03-15\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_date_lt/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_email/2","doc":"Tests whether an email is contained in text","ref":"Expression.Callbacks.Standard.html#has_email/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_email/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *email* of type **String**\n.\n\n```\n> has_email(\"my email is foo1@bar.com, please respond\")\n%{\"__value__\" => true, \"email\" => \"foo1@bar.com\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_email(\"my email is foo1@bar.com, please respond\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_email(\\\"my email is foo1@bar.com, please respond\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"email\" => \"foo1@bar.com\"} = result\n    %{\"__value__\" => true, \"email\" => \"foo1@bar.com\"}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_email(\\\"my email is foo1@bar.com, please respond\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_email/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_email/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\nfalse\n```\nwith the following fields:\n\n* *email* of type **Null**\n.\n\n```\n> has_email(\"i'm not sharing my email\")\n%{\"__value__\" => false, \"email\" => nil}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_email(\"i'm not sharing my email\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_email(\\\"i'm not sharing my email\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => false, \"email\" => nil} = result\n    %{\"__value__\" => false, \"email\" => nil}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_email(\\\"i'm not sharing my email\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_email/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_group/3","doc":"Returns whether the contact is part of group with the passed in UUID","ref":"Expression.Callbacks.Standard.html#has_group/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_group/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}\n```\n\n\n```\n> has_group(contact.groups, \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_group(contact.groups, \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_group(contact.groups, \\\"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\\\")\",\n    ...>   %{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_group(contact.groups, \\\"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\\\")\",\n    ...>   %{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_group/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_group/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}\n```\n\n\n```\n> has_group(contact.groups, \"00000000-0000-0000-0000-000000000000\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_group(contact.groups, \"00000000-0000-0000-0000-000000000000\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_group(contact.groups, \\\"00000000-0000-0000-0000-000000000000\\\")\",\n    ...>   %{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_group(contact.groups, \\\"00000000-0000-0000-0000-000000000000\\\")\",\n    ...>   %{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_group/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_number/2","doc":"Tests whether `expression` contains a number","ref":"Expression.Callbacks.Standard.html#has_number/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *number* of type **Float**\n.\n\n```\n> has_number(\"the number is 42 and 5\")\n%{\"__value__\" => true, \"number\" => 42.0}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"the number is 42 and 5\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number(\\\"the number is 42 and 5\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"number\" => 42.0} = result\n    %{\"__value__\" => true, \"number\" => 42.0}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number(\\\"the number is 42 and 5\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *number* of type **Float**\n.\n\n```\n> has_number(\"العدد ٤٢\")\n%{\"__value__\" => true, \"number\" => 42.0}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"العدد ٤٢\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number(\\\"العدد ٤٢\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"number\" => 42.0} = result\n    %{\"__value__\" => true, \"number\" => 42.0}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number(\\\"العدد ٤٢\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *number* of type **Float**\n.\n\n```\n> has_number(\"٠.٥\")\n%{\"__value__\" => true, \"number\" => 0.5}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"٠.٥\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number(\\\"٠.٥\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"number\" => 0.5} = result\n    %{\"__value__\" => true, \"number\" => 0.5}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number(\\\"٠.٥\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *number* of type **Float**\n.\n\n```\n> has_number(\"0.6\")\n%{\"__value__\" => true, \"number\" => 0.6}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"0.6\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number(\\\"0.6\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"number\" => 0.6} = result\n    %{\"__value__\" => true, \"number\" => 0.6}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number(\\\"0.6\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.has_number_eq/3","doc":"Tests whether `expression` contains a number equal to the value","ref":"Expression.Callbacks.Standard.html#has_number_eq/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42.0\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42.0\", \"42\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 42.0\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 42.0\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"the number is 40\", \"42\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 40\", \"42\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 40\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 40\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-6"},{"type":"function","title":"Example 7: - Expression.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_eq(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_eq(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_eq/3-example-7"},{"type":"function","title":"Expression.Callbacks.Standard.has_number_gt/3","doc":"Tests whether `expression` contains a number greater than min","ref":"Expression.Callbacks.Standard.html#has_number_gt/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", 40)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", 40) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", 40)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", 40)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", 40.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", 40.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", 40.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", 40.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", \"40\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", \"40\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42.0\", \"40\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42.0\", \"40\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"the number is 40\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 40\", \"40\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-6"},{"type":"function","title":"Example 7: - Expression.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gt/3-example-7"},{"type":"function","title":"Expression.Callbacks.Standard.has_number_gte/3","doc":"Tests whether `expression` contains a number greater than or equal to min","ref":"Expression.Callbacks.Standard.html#has_number_gte/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 42.0\", \"45\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42.0\", \"45\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 42.0\\\", \\\"45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 42.0\\\", \\\"45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 40\", \"45\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 40\", \"45\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 40\\\", \\\"45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 40\\\", \\\"45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-6"},{"type":"function","title":"Example 7: - Expression.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_gte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_gte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_gte/3-example-7"},{"type":"function","title":"Expression.Callbacks.Standard.has_number_lt/3","doc":"Tests whether `expression` contains a number less than max","ref":"Expression.Callbacks.Standard.html#has_number_lt/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lt(\"the number is 42\", 44)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", 44) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", 44)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", 44)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lt(\"the number is 42\", 44.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", 44.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", 44.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", 44.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 42\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", \"40\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 42.0\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42.0\", \"40\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 40\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 40\", \"40\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-6"},{"type":"function","title":"Example 7: - Expression.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lt/3-example-7"},{"type":"function","title":"Expression.Callbacks.Standard.has_number_lte/3","doc":"Tests whether `expression` contains a number less than or equal to max","ref":"Expression.Callbacks.Standard.html#has_number_lte/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", 42)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", 42.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"the number is 42.0\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42.0\", \"40\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-6"},{"type":"function","title":"Example 7: - Expression.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"response\" => 3}\n```\n\n\n```\n> has_number_lte(\"@response\", 5)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"@response\", 5) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_number_lte(\\\"@response\\\", 5)\",\n    ...>   %{\"response\" => 3},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_number_lte(\\\"@response\\\", 5)\",\n    ...>   %{\"response\" => 3},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_number_lte/3-example-7"},{"type":"function","title":"Expression.Callbacks.Standard.has_only_phrase/3","doc":"Tests whether the text contains only phrase\n\nThe phrase must be the only text in the text to match","ref":"Expression.Callbacks.Standard.html#has_only_phrase/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_phrase(\"Quick Brown\", \"quick brown\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"Quick Brown\", \"quick brown\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_phrase(\\\"Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_phrase(\\\"Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_phrase/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_phrase(\"\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"\", \"\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_phrase(\\\"\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_phrase(\\\"\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_phrase/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_only_phrase(\"The Quick Brown Fox\", \"quick brown\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"The Quick Brown Fox\", \"quick brown\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_phrase(\\\"The Quick Brown Fox\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_phrase(\\\"The Quick Brown Fox\\\", \\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_phrase/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.has_only_text/3","doc":"Returns whether two text values are equal (case sensitive). In the case that they are, it will return the text as the match.","ref":"Expression.Callbacks.Standard.html#has_only_text/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_text(\"foo\", \"foo\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"foo\", \"foo\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_text(\\\"foo\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_text(\\\"foo\\\", \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_text/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_text(\"\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"\", \"\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_text(\\\"\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_text(\\\"\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_text/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_only_text(\"foo\", \"FOO\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"foo\", \"FOO\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_only_text(\\\"foo\\\", \\\"FOO\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_only_text(\\\"foo\\\", \\\"FOO\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_only_text/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.has_pattern/3","doc":"Tests whether `expression` matches the regex pattern\n\nBoth text values are trimmed of surrounding whitespace and matching is case-insensitive.","ref":"Expression.Callbacks.Standard.html#has_pattern/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_pattern/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_pattern(\"Buy cheese please\", \"buy (\\w+)\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_pattern(\"Buy cheese please\", \"buy (\\w+)\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_pattern(\\\"Buy cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_pattern(\\\"Buy cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_pattern/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_pattern/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_pattern(\"Sell cheese please\", \"buy (\\w+)\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_pattern(\"Sell cheese please\", \"buy (\\w+)\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_pattern(\\\"Sell cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_pattern(\\\"Sell cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_pattern/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.has_phone/2","doc":"Tests whether `expresssion` contains a phone number.\nThe optional country_code argument specifies the country to use for parsing.","ref":"Expression.Callbacks.Standard.html#has_phone/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *phonenumber* of type **String**\n.\n\n```\n> has_phone(\"my number is +12067799294 thanks\")\n%{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is +12067799294 thanks\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phone(\\\"my number is +12067799294 thanks\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"} = result\n    %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phone(\\\"my number is +12067799294 thanks\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phone/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *phonenumber* of type **String**\n.\n\n```\n> has_phone(\"my number is 2067799294 thanks\", \"US\")\n%{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is 2067799294 thanks\", \"US\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phone(\\\"my number is 2067799294 thanks\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"} = result\n    %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phone(\\\"my number is 2067799294 thanks\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phone/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *phonenumber* of type **String**\n.\n\n```\n> has_phone(\"my number is 206 779 9294 thanks\", \"US\")\n%{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is 206 779 9294 thanks\", \"US\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phone(\\\"my number is 206 779 9294 thanks\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"} = result\n    %{\"__value__\" => true, \"phonenumber\" => \"+12067799294\"}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phone(\\\"my number is 206 779 9294 thanks\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phone/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\nfalse\n```\nwith the following fields:\n\n* *phonenumber* of type **Null**\n.\n\n```\n> has_phone(\"my number is none of your business\", \"US\")\n%{\"__value__\" => false, \"phonenumber\" => nil}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is none of your business\", \"US\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phone(\\\"my number is none of your business\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => false, \"phonenumber\" => nil} = result\n    %{\"__value__\" => false, \"phonenumber\" => nil}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phone(\\\"my number is none of your business\\\", \\\"US\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phone/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.has_phone/3","doc":"","ref":"Expression.Callbacks.Standard.html#has_phone/3"},{"type":"function","title":"Expression.Callbacks.Standard.has_phrase/3","doc":"Tests whether phrase is contained in `expression`\n\nThe words in the test phrase must appear in the same order with no other words in between.","ref":"Expression.Callbacks.Standard.html#has_phrase/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phrase(\"the quick brown fox\", \"brown fox\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"brown fox\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"brown fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"brown fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phrase/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_phrase(\"the quick brown fox\", \"quick fox\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"quick fox\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"quick fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"quick fox\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phrase/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phrase(\"the quick brown fox\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_phrase/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.has_text/2","doc":"Tests whether there the `expression` has any characters in it","ref":"Expression.Callbacks.Standard.html#has_text/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_text(\"quick brown\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\"quick brown\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_text(\\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_text(\\\"quick brown\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_text/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_text(\"\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\"\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_text(\\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_text(\\\"\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_text/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_text(\" \n> \")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\" \n\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_text(\\\" \\n\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_text(\\\" \\n\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_text/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_text(123)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(123) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_text(123)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_text(123)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_text/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.has_time/2","doc":"Tests whether `expression` contains a time.","ref":"Expression.Callbacks.Standard.html#has_time/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:30\")\n%{\"__value__\" => true, \"match\" => ~T[10:30:00]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:30\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_time(\\\"the time is 10:30\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:30:00]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:30:00]}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_time(\\\"the time is 10:30\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_time/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:00 pm\")\n%{\"__value__\" => true, \"match\" => ~T[10:00:00]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:00 pm\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_time(\\\"the time is 10:00 pm\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:00:00]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:00:00]}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_time(\\\"the time is 10:00 pm\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_time/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:30:45\")\n%{\"__value__\" => true, \"match\" => ~T[10:30:45]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:30:45\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_time(\\\"the time is 10:30:45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:30:45]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:30:45]}\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_time(\\\"the time is 10:30:45\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_time/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_time(\"there is no time here, just the number 25\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"there is no time here, just the number 25\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"has_time(\\\"there is no time here, just the number 25\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@has_time(\\\"there is no time here, just the number 25\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#has_time/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.hour/2","doc":"Returns only the hour of a datetime (0 to 23)","ref":"Expression.Callbacks.Standard.html#hour/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.hour/2","doc":"> Get the current hour\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `12`.\n\n```\n> hour(now())\n12\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @hour(now()) ...\"\n\"12\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"hour(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 12 = result\n    12\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@hour(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"12\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#hour/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.if_/4","doc":"Returns one value if the condition evaluates to `true`, and another value if it evaluates to `false`","ref":"Expression.Callbacks.Standard.html#if_/4"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.if_/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"Yes\"`.\n\n```\n> if true do\n>   \"Yes\"\n> else\n>   \"No\"\n> end\n> \n\"Yes\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @if(true, \"Yes\", \"No\") ...\"\n\"Yes\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"if(true, \\\"Yes\\\", \\\"No\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"Yes\" = result\n    \"Yes\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@if(true, \\\"Yes\\\", \\\"No\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"Yes\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#if_/4-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.if_/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"No\"`.\n\n```\n> # Shorthand\n> if(false, do: \"Yes\", else: \"No\")\n\"No\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @if(false, \"Yes\", \"No\") ...\"\n\"No\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"if(false, \\\"Yes\\\", \\\"No\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"No\" = result\n    \"No\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@if(false, \\\"Yes\\\", \\\"No\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"No\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#if_/4-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.isbool/2","doc":"Returns `true` if the argument is a boolean.","ref":"Expression.Callbacks.Standard.html#isbool/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isbool(true)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(true) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isbool(false)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(false) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(1)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(1) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(0)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(0) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-4"},{"type":"function","title":"Example 5: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(\"true\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(\"true\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(\\\"true\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(\\\"true\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-5"},{"type":"function","title":"Example 6: - Expression.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(\"false\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(\"false\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isbool(\\\"false\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isbool(\\\"false\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isbool/2-example-6"},{"type":"function","title":"Expression.Callbacks.Standard.isnumber/2","doc":"Returns `true` if the argument is a number.","ref":"Expression.Callbacks.Standard.html#isnumber/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(1)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(1) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isnumber(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isnumber(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isnumber/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(1.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(1.0) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isnumber(1.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isnumber(1.0)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isnumber/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(\"1.0\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(\"1.0\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isnumber(\\\"1.0\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isnumber(\\\"1.0\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isnumber/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isnumber(\"a\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(\"a\") ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isnumber(\\\"a\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isnumber(\\\"a\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isnumber/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.isstring/2","doc":"Returns `true` if the argument is a string.","ref":"Expression.Callbacks.Standard.html#isstring/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isstring(\"hello\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(\"hello\") ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isstring(\\\"hello\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isstring(\\\"hello\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isstring/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isstring(false)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(false) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isstring(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isstring(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isstring/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isstring(1)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(1) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"isstring(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@isstring(1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#isstring/2-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.left/3","doc":"Returns the first characters in a text string. This is Unicode safe.","ref":"Expression.Callbacks.Standard.html#left/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.left/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foob\"`.\n\n```\n> left(\"foobar\", 4)\n\"foob\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @left(\"foobar\", 4) ...\"\n\"foob\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"left(\\\"foobar\\\", 4)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"foob\" = result\n    \"foob\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@left(\\\"foobar\\\", 4)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"foob\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#left/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.left/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"Умерла Мадлен Олбрай\"`.\n\n```\n> left(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20)\n\"Умерла Мадлен Олбрай\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @left(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20) ...\"\n\"Умерла Мадлен Олбрай\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"left(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"Умерла Мадлен Олбрай\" = result\n    \"Умерла Мадлен Олбрай\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@left(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"Умерла Мадлен Олбрай\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#left/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.len/2","doc":"Returns the number of characters in a text string","ref":"Expression.Callbacks.Standard.html#len/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.len/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> len(\"foo\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @len(\"foo\") ...\"\n\"3\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"len(\\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 3 = result\n    3\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@len(\\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#len/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.len/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> len(\"zoë\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @len(\"zoë\") ...\"\n\"3\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"len(\\\"zoë\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 3 = result\n    3\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@len(\\\"zoë\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#len/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.lower/2","doc":"Converts a text string to lowercase","ref":"Expression.Callbacks.Standard.html#lower/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.lower/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foo bar\"`.\n\n```\n> lower(\"Foo Bar\")\n\"foo bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @lower(\"Foo Bar\") ...\"\n\"foo bar\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"lower(\\\"Foo Bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"foo bar\" = result\n    \"foo bar\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@lower(\\\"Foo Bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"foo bar\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#lower/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.map/3","doc":"map over a list of items and apply the mapper function to every item, returning\nthe result.","ref":"Expression.Callbacks.Standard.html#map/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.map/3","doc":"> Map over the range of numbers, create a date in January for every number\n\n\nWhen used in the following Stack expression it returns a  value of type **List with values Date, Date, Date**: `[\n  \"2022-01-01\",\n  \"2022-01-02\",\n  \"2022-01-03\"\n]`.\n\n```\n> map(1..3, &date(2022, 1, &1))\n[~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @map(1..3, &date(2022, 1, &1)) ...\"\n\"2022-01-012022-01-022022-01-03\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"map(1..3, &date(2022, 1, &1))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]] = result\n    [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@map(1..3, &date(2022, 1, &1))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2022-01-012022-01-022022-01-03\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#map/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.map/3","doc":"> Map over the range of numbers, multiple each by itself and return the result\n\n\nWhen used in the following Stack expression it returns a  value of type **List with values Integer, Integer, Integer**: `[\n  1,\n  4,\n  9\n]`.\n\n```\n> map(1..3, &(&1 * &1))\n[1, 4, 9]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @map(1..3, &(&1 * &1)) ...\"\n\"149\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"map(1..3, &(&1 * &1))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert [1, 4, 9] = result\n    [1, 4, 9]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@map(1..3, &(&1 * &1))\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"149\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#map/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.max_vargs/2","doc":"Returns the maximum value of all arguments","ref":"Expression.Callbacks.Standard.html#max_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.max_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> max(1, 2, 3)\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @max(1, 2, 3) ...\"\n\"3\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"max(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 3 = result\n    3\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@max(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#max_vargs/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.min_vargs/2","doc":"Returns the minimum value of all arguments","ref":"Expression.Callbacks.Standard.html#min_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.min_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> min(1, 2, 3)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @min(1, 2, 3) ...\"\n\"1\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"min(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 1 = result\n    1\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@min(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#min_vargs/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.minute/2","doc":"Returns only the minute of a datetime (0 to 59)","ref":"Expression.Callbacks.Standard.html#minute/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.minute/2","doc":"> Get the current minute\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `53`.\n\n```\n> minute(now())\n53\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @minute(now()) ...\"\n\"53\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"minute(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 53 = result\n    53\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@minute(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"53\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#minute/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.month/2","doc":"Returns only the month of a date (1 to 12)","ref":"Expression.Callbacks.Standard.html#month/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.month/2","doc":"> Get the current month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> month(now())\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @month(now()) ...\"\n\"1\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"month(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 1 = result\n    1\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@month(now())\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#month/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.not_/2","doc":"Returns `false` if the argument supplied evaluates to truth-y","ref":"Expression.Callbacks.Standard.html#not_/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.not_/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> not(false)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @not(false) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"not(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@not(false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#not_/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.now/1","doc":"Returns the current date time as UTC\n\n```\nIt is currently @NOW()\n```","ref":"Expression.Callbacks.Standard.html#now/1"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.now/1","doc":"> return the current timestamp as a DateTime value\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2024-01-16T12:53:57.303553Z\"`.\n\n```\n> now()\n~U[2024-01-16 12:53:57.303553Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @now() ...\"\n\"2024-01-16T12:53:57.303553Z\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"now()\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert ~U[2024-01-16 12:53:57.303553Z] = result\n    ~U[2024-01-16 12:53:57.303553Z]\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@now()\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2024-01-16T12:53:57.303553Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#now/1-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.now/1","doc":"> return the current datetime and format it using `datevalue`\n\n\nWhen used in the following Stack expression it returns a complex **String** type of default value:\n```\n\"2024-01-16\"\n```\nwith the following fields:\n\n* *date* of type **DateTime**\n.\n\n```\n> datevalue(now(), \"%Y-%m-%d\")\n%{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(now(), \"%Y-%m-%d\") ...\"\n\"2024-01-16\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"datevalue(now(), \\\"%Y-%m-%d\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]} = result\n    %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@datevalue(now(), \\\"%Y-%m-%d\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2024-01-16\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#now/1-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.or_vargs/2","doc":"Returns `true` if any argument is `true`.\nReturns the first truthy value found or otherwise false.\n\nAccepts any amount of arguments for testing truthiness.","ref":"Expression.Callbacks.Standard.html#or_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.or_vargs/2","doc":"> Return true if any of the values are true\n\n\nWhen used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> true or false\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(true, false) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"or(true, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@or(true, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#or_vargs/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.or_vargs/2","doc":"> Return the first value that is truthy\n\n\nWhen used in the following Stack expression it returns a  value of type **String**: `\"foo\"`.\n\n```\n> false or \"foo\"\n\"foo\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(false, \"foo\") ...\"\n\"foo\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"or(false, \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"foo\" = result\n    \"foo\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@or(false, \\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"foo\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#or_vargs/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.or_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> true or true\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(true, true) ...\"\n\"true\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"or(true, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert true = result\n    true\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@or(true, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#or_vargs/2-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.or_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> false or false\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(false, false) ...\"\n\"false\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"or(false, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    false\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@or(false, false)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#or_vargs/2-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.parse_datevalue/3","doc":"Parse random dates and times with `strftime` patterns and return a DateTime value\nwhen it matches.","ref":"Expression.Callbacks.Standard.html#parse_datevalue/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.parse_datevalue/3","doc":"> Parse a date value using strftime formatting and return a DateTime\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `\"2016-02-29T22:25:00Z\"`.\n\n```\n> parse_datevalue(\"2016-02-29T22:25:00-00:00\", \"%FT%T%:z\")\n~U[2016-02-29 22:25:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @parse_datevalue(\"2016-02-29T22:25:00-00:00\", \"%FT%T%:z\") ...\"\n\"2016-02-29T22:25:00Z\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"parse_datevalue(\\\"2016-02-29T22:25:00-00:00\\\", \\\"%FT%T%:z\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~U[2016-02-29 22:25:00Z] = result\n    ~U[2016-02-29 22:25:00Z]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@parse_datevalue(\\\"2016-02-29T22:25:00-00:00\\\", \\\"%FT%T%:z\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2016-02-29T22:25:00Z\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#parse_datevalue/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.parse_datevalue/3","doc":"> Attempt to parse a date value and return nil when failing\n\n\nWhen used in the following Stack expression it returns a  value of type **Null**: `null`.\n\n```\n> parse_datevalue(\"👻👻👻👻\", \"%FT%T%:z\")\nnil\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @parse_datevalue(\"👻👻👻👻\", \"%FT%T%:z\") ...\"\n\"\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"parse_datevalue(\\\"👻👻👻👻\\\", \\\"%FT%T%:z\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> refute result\n    nil\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@parse_datevalue(\\\"👻👻👻👻\\\", \\\"%FT%T%:z\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#parse_datevalue/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.parse_float/1","doc":"","ref":"Expression.Callbacks.Standard.html#parse_float/1"},{"type":"function","title":"Expression.Callbacks.Standard.percent/2","doc":"Formats a number as a percentage","ref":"Expression.Callbacks.Standard.html#percent/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"`.\n\n```\n> percent(2/10)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(2/10) ...\"\n\"20%\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"percent(2/10)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@percent(2/10)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#percent/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"`.\n\n```\n> percent(0.2)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(0.2) ...\"\n\"20%\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"percent(0.2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@percent(0.2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#percent/2-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"` when used with the following context:\n\n```elixir\n%{\"d\" => \"0.2\"}\n```\n\n\n```\n> percent(d)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(d) ...\"\n\"20%\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"percent(d)\",\n    ...>   %{\"d\" => \"0.2\"},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@percent(d)\",\n    ...>   %{\"d\" => \"0.2\"},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#percent/2-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.power/3","doc":"Returns the result of a number raised to a power - equivalent to the ^ operator","ref":"Expression.Callbacks.Standard.html#power/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.power/3","doc":"When used in the following Stack expression it returns a  value of type **Float**: `8.0`.\n\n```\n> power(2, 3)\n8.0\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @power(2, 3) ...\"\n\"8.0\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"power(2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 8.0 = result\n    8.0\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@power(2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"8.0\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#power/3-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.proper/2","doc":"Capitalizes the first letter of every word in a text string","ref":"Expression.Callbacks.Standard.html#proper/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.proper/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"Foo Bar\"`.\n\n```\n> proper(\"foo bar\")\n\"Foo Bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @proper(\"foo bar\") ...\"\n\"Foo Bar\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"proper(\\\"foo bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"Foo Bar\" = result\n    \"Foo Bar\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@proper(\\\"foo bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"Foo Bar\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#proper/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.read_digits/2","doc":"Formats digits in text for reading in TTS","ref":"Expression.Callbacks.Standard.html#read_digits/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.read_digits/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"plus two seven one\"`.\n\n```\n> read_digits(\"+271\")\n\"plus two seven one\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @read_digits(\"+271\") ...\"\n\"plus two seven one\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"read_digits(\\\"+271\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"plus two seven one\" = result\n    \"plus two seven one\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@read_digits(\\\"+271\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"plus two seven one\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#read_digits/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.rem/3","doc":"Return the division remainder of two integers.","ref":"Expression.Callbacks.Standard.html#rem/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.rem/3","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `0`.\n\n```\n> rem(4, 2)\n0\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rem(4, 2) ...\"\n\"0\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"rem(4, 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 0 = result\n    0\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@rem(4, 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"0\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#rem/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.rem/3","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> rem(85, 3)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rem(85, 3) ...\"\n\"1\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"rem(85, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 1 = result\n    1\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@rem(85, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#rem/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.remove_first_word/2","doc":"Removes the first word from the given text. The remaining text will be unchanged","ref":"Expression.Callbacks.Standard.html#remove_first_word/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.remove_first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"bar\"`.\n\n```\n> remove_first_word(\"foo bar\")\n\"bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @remove_first_word(\"foo bar\") ...\"\n\"bar\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"remove_first_word(\\\"foo bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"bar\" = result\n    \"bar\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@remove_first_word(\\\"foo bar\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"bar\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#remove_first_word/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.remove_first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"bar\"`.\n\n```\n> remove_first_word(\"foo-bar\", \"-\")\n\"bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @remove_first_word(\"foo-bar\", \"-\") ...\"\n\"bar\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"remove_first_word(\\\"foo-bar\\\", \\\"-\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"bar\" = result\n    \"bar\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@remove_first_word(\\\"foo-bar\\\", \\\"-\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"bar\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#remove_first_word/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.remove_first_word/3","doc":"","ref":"Expression.Callbacks.Standard.html#remove_first_word/3"},{"type":"function","title":"Expression.Callbacks.Standard.rept/3","doc":"Repeats text a given number of times","ref":"Expression.Callbacks.Standard.html#rept/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.rept/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"**********\"`.\n\n```\n> rept(\"*\", 10)\n\"**********\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rept(\"*\", 10) ...\"\n\"**********\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"rept(\\\"*\\\", 10)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"**********\" = result\n    \"**********\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@rept(\\\"*\\\", 10)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"**********\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#rept/3-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.right/3","doc":"Returns the last characters in a text string.\nThis is Unicode safe.","ref":"Expression.Callbacks.Standard.html#right/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.right/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ing\"`.\n\n```\n> right(\"testing\", 3)\n\"ing\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @right(\"testing\", 3) ...\"\n\"ing\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"right(\\\"testing\\\", 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"ing\" = result\n    \"ing\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@right(\\\"testing\\\", 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"ing\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#right/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.right/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ту главы Госдепа США\"`.\n\n```\n> right(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20)\n\"ту главы Госдепа США\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @right(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20) ...\"\n\"ту главы Госдепа США\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"right(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"ту главы Госдепа США\" = result\n    \"ту главы Госдепа США\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@right(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"ту главы Госдепа США\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#right/3-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.second/2","doc":"Returns only the second of a datetime (0 to 59)","ref":"Expression.Callbacks.Standard.html#second/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.second/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `57` when used with the following context:\n\n```elixir\n%{\"now\" => ~U[2024-01-16 12:53:57.311421Z]}\n```\n\n\n```\n> second(now)\n57\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @second(now) ...\"\n\"57\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"second(now)\",\n    ...>   %{\"now\" => ~U[2024-01-16 12:53:57.311421Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 57 = result\n    57\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@second(now)\",\n    ...>   %{\"now\" => ~U[2024-01-16 12:53:57.311421Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"57\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#second/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.substitute/4","doc":"Substitutes new_text for old_text in a text string. If instance_num is given, then only that instance will be substituted","ref":"Expression.Callbacks.Standard.html#substitute/4"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.substitute/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"I can do\"`.\n\n```\n> substitute(\"I can't\", \"can't\", \"can do\")\n\"I can do\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @substitute(\"I can't\", \"can't\", \"can do\") ...\"\n\"I can do\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"substitute(\\\"I can't\\\", \\\"can't\\\", \\\"can do\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"I can do\" = result\n    \"I can do\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@substitute(\\\"I can't\\\", \\\"can't\\\", \\\"can do\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"I can do\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#substitute/4-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.sum_vargs/2","doc":"Returns the sum of all arguments, equivalent to the + operator\n\n```\nYou have @SUM(contact.reports, contact.forms) reports and forms\n```","ref":"Expression.Callbacks.Standard.html#sum_vargs/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.sum_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `6`.\n\n```\n> sum(1, 2, 3)\n6\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @sum(1, 2, 3) ...\"\n\"6\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"sum(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 6 = result\n    6\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@sum(1, 2, 3)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"6\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#sum_vargs/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.time/4","doc":"Defines a time value which can be used for time arithmetic","ref":"Expression.Callbacks.Standard.html#time/4"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.time/4","doc":"When used in the following Stack expression it returns a  value of type **Time**: `\"12:13:14\"`.\n\n```\n> time(12, 13, 14)\n~T[12:13:14]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @time(12, 13, 14) ...\"\n\"~T[12:13:14]\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"time(12, 13, 14)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~T[12:13:14] = result\n    ~T[12:13:14]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@time(12, 13, 14)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"~T[12:13:14]\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#time/4-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.timevalue/2","doc":"Converts time stored in text to an actual time","ref":"Expression.Callbacks.Standard.html#timevalue/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.timevalue/2","doc":"When used in the following Stack expression it returns a  value of type **Time**: `\"02:30:00\"`.\n\n```\n> timevalue(\"2:30\")\n~T[02:30:00]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @timevalue(\"2:30\") ...\"\n\"~T[02:30:00]\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"timevalue(\\\"2:30\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~T[02:30:00] = result\n    ~T[02:30:00]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@timevalue(\\\"2:30\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"~T[02:30:00]\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#timevalue/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.timevalue/2","doc":"When used in the following Stack expression it returns a  value of type **Time**: `\"02:30:55\"`.\n\n```\n> timevalue(\"2:30:55\")\n~T[02:30:55]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @timevalue(\"2:30:55\") ...\"\n\"~T[02:30:55]\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"timevalue(\\\"2:30:55\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert ~T[02:30:55] = result\n    ~T[02:30:55]\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@timevalue(\\\"2:30:55\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"~T[02:30:55]\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#timevalue/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.today/1","doc":"Returns the current date","ref":"Expression.Callbacks.Standard.html#today/1"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.today/1","doc":"When used in the following Stack expression it returns a  value of type **Date**: `\"2024-01-16\"`.\n\n```\n> today()\n~D[2024-01-16]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @today() ...\"\n\"2024-01-16\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"today()\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert ~D[2024-01-16] = result\n    ~D[2024-01-16]\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@today()\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2024-01-16\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#today/1-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.unichar/2","doc":"Returns the unicode character specified by a number","ref":"Expression.Callbacks.Standard.html#unichar/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.unichar/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"A\"`.\n\n```\n> unichar(65)\n\"A\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unichar(65) ...\"\n\"A\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"unichar(65)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"A\" = result\n    \"A\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@unichar(65)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"A\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#unichar/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.unichar/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"é\"`.\n\n```\n> unichar(233)\n\"é\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unichar(233) ...\"\n\"é\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"unichar(233)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"é\" = result\n    \"é\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@unichar(233)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"é\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#unichar/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.unicode/2","doc":"Returns a numeric code for the first character in a text string","ref":"Expression.Callbacks.Standard.html#unicode/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.unicode/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `65`.\n\n```\n> unicode(\"A\")\n65\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unicode(\"A\") ...\"\n\"65\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"unicode(\\\"A\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 65 = result\n    65\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@unicode(\\\"A\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"65\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#unicode/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.unicode/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `233`.\n\n```\n> unicode(\"é\")\n233\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unicode(\"é\") ...\"\n\"233\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"unicode(\\\"é\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 233 = result\n    233\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@unicode(\\\"é\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"233\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#unicode/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.upper/2","doc":"Converts a text string to uppercase","ref":"Expression.Callbacks.Standard.html#upper/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.upper/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"FOO\"`.\n\n```\n> upper(\"foo\")\n\"FOO\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @upper(\"foo\") ...\"\n\"FOO\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"upper(\\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"FOO\" = result\n    \"FOO\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@upper(\\\"foo\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"FOO\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#upper/2-example-1"},{"type":"function","title":"Expression.Callbacks.Standard.weekday/2","doc":"Returns the day of the week of a date (1 for Sunday to 7 for Saturday)","ref":"Expression.Callbacks.Standard.html#weekday/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.weekday/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1` when used with the following context:\n\n```elixir\n%{\"today\" => ~D[2022-11-06]}\n```\n\n\n```\n> weekday(today)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @weekday(today) ...\"\n\"1\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"weekday(today)\",\n    ...>   %{\"today\" => ~D[2022-11-06]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 1 = result\n    1\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@weekday(today)\",\n    ...>   %{\"today\" => ~D[2022-11-06]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#weekday/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.weekday/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3` when used with the following context:\n\n```elixir\n%{\"today\" => ~D[2022-11-01]}\n```\n\n\n```\n> weekday(today)\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @weekday(today) ...\"\n\"3\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"weekday(today)\",\n    ...>   %{\"today\" => ~D[2022-11-01]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 3 = result\n    3\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@weekday(today)\",\n    ...>   %{\"today\" => ~D[2022-11-01]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#weekday/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.word/3","doc":"Extracts the nth word from the given text string. If stop is a negative number,\nthen it is treated as count backwards from the end of the text. If by_spaces is\nspecified and is `true` then the function splits the text into words only by spaces.\nOtherwise the text is split by punctuation characters as well","ref":"Expression.Callbacks.Standard.html#word/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"cow\"`.\n\n```\n> word(\"hello cow-boy\", 2)\n\"cow\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", 2) ...\"\n\"cow\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word(\\\"hello cow-boy\\\", 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"cow\" = result\n    \"cow\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word(\\\"hello cow-boy\\\", 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"cow\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"cow-boy\"`.\n\n```\n> word(\"hello cow-boy\", 2, true)\n\"cow-boy\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", 2, true) ...\"\n\"cow-boy\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word(\\\"hello cow-boy\\\", 2, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"cow-boy\" = result\n    \"cow-boy\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word(\\\"hello cow-boy\\\", 2, true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"cow-boy\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"boy\"`.\n\n```\n> word(\"hello cow-boy\", -1)\n\"boy\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", -1) ...\"\n\"boy\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word(\\\"hello cow-boy\\\", -1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"boy\" = result\n    \"boy\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word(\\\"hello cow-boy\\\", -1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"boy\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word/3-example-3"},{"type":"function","title":"Expression.Callbacks.Standard.word/4","doc":"","ref":"Expression.Callbacks.Standard.html#word/4"},{"type":"function","title":"Expression.Callbacks.Standard.word_count/2","doc":"Returns the number of words in the given text string. If by_spaces is specified and is `true` then the function splits the text into words only by spaces. Otherwise the text is split by punctuation characters as well\n\n```\n> You entered @word_count(\"one two three\") words\nYou entered 3 words\n```","ref":"Expression.Callbacks.Standard.html#word_count/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.word_count/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> word_count(\"hello cow-boy\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_count(\"hello cow-boy\") ...\"\n\"3\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_count(\\\"hello cow-boy\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 3 = result\n    3\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_count(\\\"hello cow-boy\\\")\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_count/2-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.word_count/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `2`.\n\n```\n> word_count(\"hello cow-boy\", true)\n2\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_count(\"hello cow-boy\", true) ...\"\n\"2\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_count(\\\"hello cow-boy\\\", true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert 2 = result\n    2\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_count(\\\"hello cow-boy\\\", true)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_count/2-example-2"},{"type":"function","title":"Expression.Callbacks.Standard.word_count/3","doc":"","ref":"Expression.Callbacks.Standard.html#word_count/3"},{"type":"function","title":"Expression.Callbacks.Standard.word_slice/3","doc":"Extracts a substring of the words beginning at start, and up to but not-including stop.\nIf stop is omitted then the substring will be all words from start until the end of the text.\nIf stop is a negative number, then it is treated as count backwards from the end of the text.\nIf by_spaces is specified and is `true` then the function splits the text into words only by spaces.\nOtherwise the text is split by punctuation characters as well","ref":"Expression.Callbacks.Standard.html#word_slice/3"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"expressions are\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 2, 4)\n\"expressions are\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 2, 4) ...\"\n\"expressions are\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 2, 4)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"expressions are\" = result\n    \"expressions are\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 2, 4)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"expressions are\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_slice/3-example-1"},{"type":"function","title":"Example 2: - Expression.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"expressions are fun\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 2)\n\"expressions are fun\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 2) ...\"\n\"expressions are fun\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"expressions are fun\" = result\n    \"expressions are fun\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"expressions are fun\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_slice/3-example-2"},{"type":"function","title":"Example 3: - Expression.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"FLOIP expressions\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 1, -2)\n\"FLOIP expressions\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 1, -2) ...\"\n\"FLOIP expressions\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 1, -2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"FLOIP expressions\" = result\n    \"FLOIP expressions\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 1, -2)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"FLOIP expressions\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_slice/3-example-3"},{"type":"function","title":"Example 4: - Expression.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"fun\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", -1)\n\"fun\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", -1) ...\"\n\"fun\"\n```\n\n    iex> import ExUnit.Assertions\n    iex> result = Expression.evaluate_block!(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", -1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    iex> assert \"fun\" = result\n    \"fun\"\n    iex> Expression.evaluate_as_string!(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", -1)\",\n    ...>   %{},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"fun\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#word_slice/3-example-4"},{"type":"function","title":"Expression.Callbacks.Standard.word_slice/4","doc":"","ref":"Expression.Callbacks.Standard.html#word_slice/4"},{"type":"function","title":"Expression.Callbacks.Standard.word_slice/5","doc":"","ref":"Expression.Callbacks.Standard.html#word_slice/5"},{"type":"function","title":"Expression.Callbacks.Standard.year/2","doc":"Returns only the year of a date","ref":"Expression.Callbacks.Standard.html#year/2"},{"type":"function","title":"Example 1: - Expression.Callbacks.Standard.year/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `2024` when used with the following context:\n\n```elixir\n%{\"now\" => ~U[2024-01-16 12:53:57.312494Z]}\n```\n\n\n```\n> year(now)\n2024\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @year(now) ...\"\n\"2024\"\n```\n\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.evaluate_block!(\n    ...>   \"year(now)\",\n    ...>   %{\"now\" => ~U[2024-01-16 12:53:57.312494Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    ..$> assert 2024 = result\n    2024\n    ..$> Expression.evaluate_as_string!(\n    ...>   \"@year(now)\",\n    ...>   %{\"now\" => ~U[2024-01-16 12:53:57.312494Z]},\n    ...>   Expression.Callbacks.Standard\n    ...> )\n    \"2024\"\n\n\n---","ref":"Expression.Callbacks.Standard.html#year/2-example-1"},{"type":"module","title":"Expression.Context","doc":"A helper module for creating a context that can be\nused with Expression.Eval\n\n# Example\n\n  iex> Expression.Context.new(%{foo: \"bar\"})\n  %{\"foo\" => \"bar\"}\n  iex> Expression.Context.new(%{FOO: \"bar\"})\n  %{\"foo\" => \"bar\"}\n  iex> Expression.Context.new(%{foo: %{bar: \"baz\"}})\n  %{\"foo\" => %{\"bar\" => \"baz\"}}\n  iex> Expression.Context.new(%{Foo: %{Bar: \"baz\"}})\n  %{\"foo\" => %{\"bar\" => \"baz\"}}\n  iex> Expression.Context.new(%{foo: %{bar: 1}})\n  %{\"foo\" => %{\"bar\" => 1}}\n  iex> Expression.Context.new(%{date: \"2020-12-13T23:34:45\"})\n  %{\"date\" => ~U[2020-12-13 23:34:45.0Z]}\n  iex> Expression.Context.new(%{boolean: \"true\"})\n  %{\"boolean\" => true}\n  iex> Expression.Context.new(%{float: 1.234})\n  %{\"float\" => 1.234}\n  iex> now = DateTime.utc_now()\n  iex> ctx = Expression.Context.new(%{float: \"1.234\", nested: %{date: now}})\n  iex> ctx[\"float\"]\n  1.234\n  iex> now == ctx[\"nested\"][\"date\"]\n  true\n  iex> Expression.Context.new(%{mixed: [\"2020-12-13T23:34:45\", 1, \"true\", \"binary\"]})\n  %{\"mixed\" => [~U[2020-12-13 23:34:45.0Z], 1, true, \"binary\"]}","ref":"Expression.Context.html"},{"type":"function","title":"Expression.Context.new/1","doc":"","ref":"Expression.Context.html#new/1"},{"type":"type","title":"Expression.Context.t/0","doc":"","ref":"Expression.Context.html#t:t/0"},{"type":"module","title":"Expression.Eval","doc":"Expression.Eval is responsible for taking an abstract syntax\ntree (AST) as generated by Expression.Parser and evaluating it.\n\nAt a high level, an AST consists of a Keyword list with two top-level\nkeys, either `:text` or `:expression`.\n\n`Expression.Eval.eval!/3` will return the output for each entry in the Keyword\nlist. `:text` entries are returned as regular strings. `:expression` entries\nare returned as typed values.\n\nThe returned value is a list containing each.\n\n# Example\n\n    iex(1)> Expression.Eval.eval!([text: \"hello\"], %{})\n    [\"hello\"]\n    iex(2)> Expression.Eval.eval!([text: \"hello\", expression: [literal: 1]], %{})\n    [\"hello\", 1]\n    iex(3)> Expression.Eval.eval!([\n    ...(3)>   text: \"hello\",\n    ...(3)>   expression: [literal: 1],\n    ...(3)>   text: \"ok\",\n    ...(3)>   expression: [literal: true]\n    ...(3)> ], %{})\n    [\"hello\", 1, \"ok\", true]","ref":"Expression.Eval.html"},{"type":"function","title":"Expression.Eval.default_value/2","doc":"Return the default value for a potentially complex value.\n\nComplex values can be Maps that have a `__value__` key, if that's\nreturned then we can to use the `__value__` value when eval'ing against\noperators or functions.","ref":"Expression.Eval.html#default_value/2"},{"type":"function","title":"Expression.Eval.eval!/3","doc":"","ref":"Expression.Eval.html#eval!/3"},{"type":"function","title":"Expression.Eval.handle_not_found/1","doc":"","ref":"Expression.Eval.html#handle_not_found/1"},{"type":"function","title":"Expression.Eval.not_founds_as_nil/1","doc":"","ref":"Expression.Eval.html#not_founds_as_nil/1"},{"type":"function","title":"Expression.Eval.op/3","doc":"","ref":"Expression.Eval.html#op/3"},{"type":"module","title":"Expression.Parser","doc":"Expression.Parser is responsible for accepting a string\ncontaining an expression and returning the abstract syntax\ntree (AST) representing the expression.\n\nThe AST generated by this module can be evaluated by\nExpression.Eval\n\n# Example\n\n  iex(1)> Expression.Parser.parse(\"hello @world\")\n  {:ok, [text: \"hello \", expression: [atom: \"world\"]], \"\", %{}, {1, 0}, 12}","ref":"Expression.Parser.html"},{"type":"function","title":"Expression.Parser.aexpr/2","doc":"Parses the given `binary` as aexpr.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the aexpr (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#aexpr/2"},{"type":"function","title":"Options - Expression.Parser.aexpr/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#aexpr/2-options"},{"type":"function","title":"Expression.Parser.aexpr_exponent/2","doc":"Parses the given `binary` as aexpr_exponent.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the aexpr_exponent (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#aexpr_exponent/2"},{"type":"function","title":"Options - Expression.Parser.aexpr_exponent/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#aexpr_exponent/2-options"},{"type":"function","title":"Expression.Parser.aexpr_factor/2","doc":"Parses the given `binary` as aexpr_factor.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the aexpr_factor (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#aexpr_factor/2"},{"type":"function","title":"Options - Expression.Parser.aexpr_factor/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#aexpr_factor/2-options"},{"type":"function","title":"Expression.Parser.aexpr_term/2","doc":"Parses the given `binary` as aexpr_term.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the aexpr_term (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#aexpr_term/2"},{"type":"function","title":"Options - Expression.Parser.aexpr_term/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#aexpr_term/2-options"},{"type":"function","title":"Expression.Parser.arguments/2","doc":"Parses the given `binary` as arguments.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the arguments (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#arguments/2"},{"type":"function","title":"Options - Expression.Parser.arguments/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#arguments/2-options"},{"type":"function","title":"Expression.Parser.attribute/2","doc":"Parses the given `binary` as attribute.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the attribute (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#attribute/2"},{"type":"function","title":"Options - Expression.Parser.attribute/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#attribute/2-options"},{"type":"function","title":"Expression.Parser.fold_infixl/1","doc":"","ref":"Expression.Parser.html#fold_infixl/1"},{"type":"function","title":"Expression.Parser.function/2","doc":"Parses the given `binary` as function.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the function (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#function/2"},{"type":"function","title":"Options - Expression.Parser.function/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#function/2-options"},{"type":"function","title":"Expression.Parser.key/2","doc":"Parses the given `binary` as key.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the key (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#key/2"},{"type":"function","title":"Options - Expression.Parser.key/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#key/2-options"},{"type":"function","title":"Expression.Parser.lambda/2","doc":"Parses the given `binary` as lambda.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the lambda (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#lambda/2"},{"type":"function","title":"Options - Expression.Parser.lambda/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#lambda/2-options"},{"type":"function","title":"Expression.Parser.list/2","doc":"Parses the given `binary` as list.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the list (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#list/2"},{"type":"function","title":"Options - Expression.Parser.list/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#list/2-options"},{"type":"function","title":"Expression.Parser.literal/2","doc":"Parses the given `binary` as literal.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the literal (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#literal/2"},{"type":"function","title":"Options - Expression.Parser.literal/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#literal/2-options"},{"type":"function","title":"Expression.Parser.parse/2","doc":"Parses the given `binary` as parse.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the parse (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#parse/2"},{"type":"function","title":"Options - Expression.Parser.parse/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#parse/2-options"},{"type":"function","title":"Expression.Parser.variable/2","doc":"Parses the given `binary` as variable.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the variable (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Expression.Parser.html#variable/2"},{"type":"function","title":"Options - Expression.Parser.variable/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Expression.Parser.html#variable/2-options"},{"type":"module","title":"Expression.V2","doc":"A second attempt at the parser, hopefully a little easier to read & maintain.\n\n`parse/1` parsed an Expression into AST.\n`eval/3` evaluates the given AST using the context supplied.\n\nFor details on how this is done please read `Expression.V2.Parser` and\n`Expression.V2.Compile`.\n\nThis parser & evaluator supports the following:\n\n* [strings](https://hexdocs.pm/elixir/typespecs.html#basic-types) either double or single quoted.\n* [integers](https://hexdocs.pm/elixir/typespecs.html#basic-types) such as `1`, `2`, `40`, `55`\n* [floats](https://hexdocs.pm/elixir/typespecs.html#basic-types) such as `3.141592653589793`\n* [booleans](https://hexdocs.pm/elixir/typespecs.html#basic-types) which can be written in any mixed case such as `tRue` or `TRUE`, `False` etc\n* `Range.t` such as `1..10`, also with steps `1..10//2`\n* `Date.t` such as `2022-01-01` which is parsed into `~D[2022-01-01]`\n* `Time.t` such as `10:30` which is parsed into `~T[10:30:00]`\n* ISO formatted `DateTime.t` such as `2022-05-24T00:00:00` which is parsed into `~U[2022-05-24 00:00:00.0Z]`\n* US formatted `DateTime.t` such as `01-02-2020 23:23:23` which is parsed into `~U[2020-02-01T23:23:23Z]`\n* Lists of any of the above, such as `[1, 2, 3]` or `[1, 1.234, \"john\"]`\n* Reading properties off of nested objects such as maps with a full stop, such as `contact.name` returning `\"Doe\"` from `%{\"contact\" => %{\"name\" => \"Doe\"}}`\n* Reading attributes off of maps, such as `contact[the_key]` which returns `\"Doe\"` from `%{\"contact\" => %{\"name\" => \"Doe\"}, \"the_key\" => \"name\"}`\n* Anonymous functions with `&` and `&1` as capture operators, `&(&1 + 1)` is an anonymous function that increments the input by 1.\n\nThe result of a call to `eval/3` is a list of typed evaluated items. It is up to the integrating library to determine how\nbest to convert these into a final end user representation.\n\n# Examples\n\n    iex> alias Expression.V2\n    iex> V2.eval(\"the date is @date(2022, 2, 20)\")\n    [\"the date is \", ~D[2022-02-20]]\n    iex> V2.eval(\"the answer is @true\")\n    [\"the answer is \", true]\n    iex> V2.eval(\"22 divided by 7 is @(22 / 7)\")\n    [\"22 divided by 7 is \", 3.142857142857143]\n    iex> V2.eval(\n    ...>   \"Hello @proper(contact.name)! Looking forward to meet you @date(2023, 2, 20)\",\n    ...>   V2.Context.new(%{\"contact\" => %{\"name\" => \"mary\"}})\n    ...> )\n    [\"Hello \", \"Mary\", \"! Looking forward to meet you \", ~D[2023-02-20]]\n    iex> V2.eval(\"@map(1..3, &date(2023, 1, &1))\")\n    [[~D[2023-01-01], ~D[2023-01-02], ~D[2023-01-03]]]\n    iex> V2.eval(\n    ...>   \"Here is the multiplication table of @number: @(map(1..10, &(&1 * number)))\",\n    ...>   V2.Context.new(%{\"number\" => 5})\n    ...> )\n    [\n      \"Here is the multiplication table of \",\n      5,\n      \": \",\n      [5, 10, 15, 20, 25, 30, 35, 40, 45, 50]\n    ]","ref":"Expression.V2.html"},{"type":"function","title":"Expression.V2.compile/1","doc":"","ref":"Expression.V2.html#compile/1"},{"type":"function","title":"Expression.V2.compile_block/1","doc":"","ref":"Expression.V2.html#compile_block/1"},{"type":"function","title":"Expression.V2.debug/1","doc":"Return the code generated for the Abstract Syntax tree or\nExpression string provided.","ref":"Expression.V2.html#debug/1"},{"type":"function","title":"Expression.V2.default_value/2","doc":"Return the default value for a potentially complex value.\n\nComplex values can be Maps that have a `__value__` key, if that's\nreturned then we can to use the `__value__` value when eval'ing against\noperators or functions.","ref":"Expression.V2.html#default_value/2"},{"type":"function","title":"Expression.V2.escape/1","doc":"","ref":"Expression.V2.html#escape/1"},{"type":"function","title":"Expression.V2.eval/2","doc":"Evaluate a string with expressions against a given context","ref":"Expression.V2.html#eval/2"},{"type":"function","title":"Expression.V2.eval_as_string/2","doc":"Evaluate an expression and cast all items to strings before joining\nthe full result into a single string value to be returned.\n\nThis calls `eval/2` internally, maps the results with `default_value/2`\nfollowed by `stringify/1` and then joins them.","ref":"Expression.V2.html#eval_as_string/2"},{"type":"function","title":"Expression.V2.eval_ast/2","doc":"Evaluate a parsed Expression against a given context","ref":"Expression.V2.html#eval_ast/2"},{"type":"function","title":"Expression.V2.eval_block/2","doc":"Evaluate a string with an expression block against a context","ref":"Expression.V2.html#eval_block/2"},{"type":"function","title":"Expression.V2.eval_block_ast/2","doc":"Evaluate the given AST against a given context","ref":"Expression.V2.html#eval_block_ast/2"},{"type":"function","title":"Expression.V2.parse/1","doc":"Parse a string with expressions into AST for the compile step","ref":"Expression.V2.html#parse/1"},{"type":"function","title":"Expression.V2.parse_block/1","doc":"Parse a string with an expression block into AST for the compile step","ref":"Expression.V2.html#parse_block/1"},{"type":"function","title":"Expression.V2.read_attribute/2","doc":"This function is referenced by `Expression.V2.Compile` to\nmake access to values in Maps or Lists easier","ref":"Expression.V2.html#read_attribute/2"},{"type":"function","title":"Expression.V2.stringify/1","doc":"","ref":"Expression.V2.html#stringify/1"},{"type":"module","title":"Expression.V2.Autodoc","doc":"Extract `@expression_doc` attributes from modules defining callbacks\nand automatically write doctests for those.\n\nAlso inserts an `expression_docs()` function which returns a list of\nall functions and their defined expression docs.\n\nThe format is:\n\n```elixir\n@expression_doc doc: \"Construct a date from year, month, and day integers\",\n                expression: \"@date(year, month, day)\",\n                context: %{\"year\" => 2022, \"month\" => 1, \"day\" => 31},\n                result: \"2022-01-31T00:00:00Z\"\n```\n\nWhere:\n\n* `doc` is the explanatory text added to the doctest.\n* `expression` is the expression we want to test\n* `fake_expression` can optionally be the expression we want to display but not test\n* `context` is the context the expression is tested against\n* `result` is the result we're expecting to get and are asserting against\n* `fake_result` can be optionally supplied when the returning result varies\n   depending on factors we do not control, like for `now()` for example.\n   When this is used, the ExDoc tests are faked and won't actually test\n   anything so use sparingly.","ref":"Expression.V2.Autodoc.html"},{"type":"function","title":"Expression.V2.Autodoc.annotate_method/3","doc":"","ref":"Expression.V2.Autodoc.html#annotate_method/3"},{"type":"function","title":"Expression.V2.Autodoc.format_context/1","doc":"","ref":"Expression.V2.Autodoc.html#format_context/1"},{"type":"function","title":"Expression.V2.Autodoc.format_docs/1","doc":"","ref":"Expression.V2.Autodoc.html#format_docs/1"},{"type":"function","title":"Expression.V2.Autodoc.format_function_args/1","doc":"","ref":"Expression.V2.Autodoc.html#format_function_args/1"},{"type":"function","title":"Expression.V2.Autodoc.format_function_name/1","doc":"","ref":"Expression.V2.Autodoc.html#format_function_name/1"},{"type":"function","title":"Expression.V2.Autodoc.format_result/1","doc":"","ref":"Expression.V2.Autodoc.html#format_result/1"},{"type":"function","title":"Expression.V2.Autodoc.generate_assert/2","doc":"","ref":"Expression.V2.Autodoc.html#generate_assert/2"},{"type":"function","title":"Expression.V2.Autodoc.generate_ex_doc/5","doc":"","ref":"Expression.V2.Autodoc.html#generate_ex_doc/5"},{"type":"function","title":"Expression.V2.Autodoc.get_existing_docstring/1","doc":"","ref":"Expression.V2.Autodoc.html#get_existing_docstring/1"},{"type":"function","title":"Expression.V2.Autodoc.get_expression/1","doc":"","ref":"Expression.V2.Autodoc.html#get_expression/1"},{"type":"function","title":"Expression.V2.Autodoc.stringify/1","doc":"","ref":"Expression.V2.Autodoc.html#stringify/1"},{"type":"function","title":"Expression.V2.Autodoc.type_of/1","doc":"","ref":"Expression.V2.Autodoc.html#type_of/1"},{"type":"function","title":"Expression.V2.Autodoc.update_annotations/4","doc":"","ref":"Expression.V2.Autodoc.html#update_annotations/4"},{"type":"module","title":"Expression.V2.Callbacks","doc":"Use this module to implement one's own callbacks.\nThe standard callbacks available are implemented in `Expression.V2.Callbacks.Standard`.\n\n```elixir\ndefmodule MyCallbacks do\n  use Expression.V2.Callbacks\n\n  @doc \"\"\"\n  Roll a dice and randomly return a number between 1 and 6.\n  \"\"\"\n  def dice_roll(ctx) do\n    Enum.random(1..6)\n  end\n\nend\n```","ref":"Expression.V2.Callbacks.html"},{"type":"function","title":"Expression.V2.Callbacks.atom_function_name/1","doc":"Convert a string function name into an atom meant to handle\nthat function\n\nReserved words such as `and`, `if`, and `or` are automatically suffixed\nwith an `_` underscore.","ref":"Expression.V2.Callbacks.html#atom_function_name/1"},{"type":"function","title":"Expression.V2.Callbacks.callback/4","doc":"Callback a function while evaluating the context against an expression.\n\nCallback functions in this module are either:\n\n1. The function name as is\n2. The function name with an underscore suffix if the function name is a reserved word\n3. The function name suffixed with `_vargs` if the takes a variable set of arguments","ref":"Expression.V2.Callbacks.html#callback/4"},{"type":"function","title":"Expression.V2.Callbacks.implements/3","doc":"","ref":"Expression.V2.Callbacks.html#implements/3"},{"type":"module","title":"Expression.V2.Callbacks.Standard","doc":"Callback functions to be used in Expressions.\n\nThis is the same idea as `Expression.Callbacks.Standard` but\nit's in a rough shape, mostly to just prove that this all works.","ref":"Expression.V2.Callbacks.Standard.html"},{"type":"function","title":"Expression.V2.Callbacks.Standard.abs/2","doc":"Returns the absolute value of a number","ref":"Expression.V2.Callbacks.Standard.html#abs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.abs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> abs(-1)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @abs(-1) ...\"\n\"1\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @abs(-1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 1, \" impact\"] = result\n    [\"chat for \", 1, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"abs(-1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 1 = result\n    1\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@abs(-1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#abs/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.and_vargs/2","doc":"Returns `true` if and only if all its arguments evaluate to `true`","ref":"Expression.V2.Callbacks.Standard.html#and_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.and_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}}\n```\n\n\n```\n> contact.gender = \"F\" and contact.age >= 18\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @and(contact.gender = \"F\", contact.age >= 18) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @and(contact.gender = \\\"F\\\", contact.age >= 18) impact\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"F\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#and_vargs/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.and_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}}\n```\n\n\n```\n> contact.gender = \"F\" and contact.age >= 18\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @and(contact.gender = \"F\", contact.age >= 18) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @and(contact.gender = \\\"F\\\", contact.age >= 18) impact\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@and(contact.gender = \\\"F\\\", contact.age >= 18)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#and_vargs/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.append/3","doc":"Appends an item or a list of items to a given list.","ref":"Expression.V2.Callbacks.Standard.html#append/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.append/3","doc":"When used in the following Stack expression it returns a  value of type **List with values String, String, String**: `[\"A\", \"B\", \"C\"]`.\n\n```\n> append([\"A\", \"B\"], \"C\")\n[\"A\", \"B\", \"C\"]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @append([\"A\", \"B\"], \"C\") ...\"\n\"ABC\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @append([\\\"A\\\", \\\"B\\\"], \\\"C\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", [\"A\", \"B\", \"C\"], \" impact\"] = result\n    [\"chat for \", [\"A\", \"B\", \"C\"], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"append([\\\"A\\\", \\\"B\\\"], \\\"C\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert [\"A\", \"B\", \"C\"] = result\n    [\"A\", \"B\", \"C\"]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@append([\\\"A\\\", \\\"B\\\"], \\\"C\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"ABC\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#append/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.append/3","doc":"When used in the following Stack expression it returns a  value of type **List with values String, String, String, String**: `[\"A\", \"B\", \"C\", \"B\"]`.\n\n```\n> append([\"A\", \"B\"], [\"C\", \"B\"])\n[\"A\", \"B\", \"C\", \"B\"]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @append([\"A\", \"B\"], [\"C\", \"B\"]) ...\"\n\"ABCB\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @append([\\\"A\\\", \\\"B\\\"], [\\\"C\\\", \\\"B\\\"]) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", [\"A\", \"B\", \"C\", \"B\"], \" impact\"] = result\n    [\"chat for \", [\"A\", \"B\", \"C\", \"B\"], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"append([\\\"A\\\", \\\"B\\\"], [\\\"C\\\", \\\"B\\\"])\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert [\"A\", \"B\", \"C\", \"B\"] = result\n    [\"A\", \"B\", \"C\", \"B\"]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@append([\\\"A\\\", \\\"B\\\"], [\\\"C\\\", \\\"B\\\"])\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"ABCB\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#append/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.callback/4","doc":"","ref":"Expression.V2.Callbacks.Standard.html#callback/4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.char/2","doc":"Returns the character specified by a number\n\n```\n> \"As easy as @char(65), @char(66), @char(67)\"\n\"As easy as A, B, C\"\n```","ref":"Expression.V2.Callbacks.Standard.html#char/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.char/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"A\"`.\n\n```\n> char(65)\n\"A\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @char(65) ...\"\n\"A\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @char(65) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"A\", \" impact\"] = result\n    [\"chat for \", \"A\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"char(65)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"A\" = result\n    \"A\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@char(65)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"A\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#char/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.clean/2","doc":"Removes all non-printable characters from a text string","ref":"Expression.V2.Callbacks.Standard.html#clean/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.clean/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ABC\"` when used with the following context:\n\n```elixir\n%{\"value\" => <<65, 0, 66, 0, 67>>}\n```\n\n\n```\n> clean(value)\n\"ABC\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @clean(value) ...\"\n\"ABC\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @clean(value) impact\",\n    ...>   Expression.V2.Context.new(%{\"value\" => <<65, 0, 66, 0, 67>>}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"ABC\", \" impact\"] = result\n    [\"chat for \", \"ABC\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"clean(value)\",\n    ...>   Expression.V2.Context.new(%{\"value\" => <<65, 0, 66, 0, 67>>}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"ABC\" = result\n    \"ABC\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@clean(value)\",\n    ...>   Expression.V2.Context.new(%{\"value\" => <<65, 0, 66, 0, 67>>}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"ABC\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#clean/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.code/2","doc":"Returns a numeric code for the first character in a text string\n\n```\n> \"The numeric code of A is @CODE(\\\"A\\\")\"\n\"The numeric code of A is 65\"\n```","ref":"Expression.V2.Callbacks.Standard.html#code/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.code/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `65`.\n\n```\n> code(\"A\")\n65\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @code(\"A\") ...\"\n\"65\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @code(\\\"A\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 65, \" impact\"] = result\n    [\"chat for \", 65, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"code(\\\"A\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 65 = result\n    65\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@code(\\\"A\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"65\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#code/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.concatenate_vargs/2","doc":"Joins text strings into one text string\n\n```\n> \"Your name is @CONCATENATE(contact.first_name, \\\" \\\", contact.last_name)\"\n\"Your name is name surname\"\n```","ref":"Expression.V2.Callbacks.Standard.html#concatenate_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.concatenate_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"name surname\"` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}}\n```\n\n\n```\n> concatenate(contact.first_name, \" \", contact.last_name)\n\"name surname\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @concatenate(contact.first_name, \" \", contact.last_name) ...\"\n\"name surname\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @concatenate(contact.first_name, \\\" \\\", contact.last_name) impact\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"name surname\", \" impact\"] = result\n    [\"chat for \", \"name surname\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"concatenate(contact.first_name, \\\" \\\", contact.last_name)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"name surname\" = result\n    \"name surname\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@concatenate(contact.first_name, \\\" \\\", contact.last_name)\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"first_name\" => \"name\", \"last_name\" => \"surname\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"name surname\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#concatenate_vargs/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.date/4","doc":"Defines a new date value","ref":"Expression.V2.Callbacks.Standard.html#date/4"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.date/4","doc":"> Construct a date from year, month, and day integers\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `~D[2022-01-31]` when used with the following context:\n\n```elixir\n%{\"day\" => 31, \"month\" => 1, \"year\" => 2022}\n```\n\n\n```\n> date(year, month, day)\n~D[2022-01-31]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @date(year, month, day) ...\"\n\"2022-01-31\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @date(year, month, day) impact\",\n    ...>   Expression.V2.Context.new(%{\"day\" => 31, \"month\" => 1, \"year\" => 2022}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~D[2022-01-31], \" impact\"] = result\n    [\"chat for \", ~D[2022-01-31], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"date(year, month, day)\",\n    ...>   Expression.V2.Context.new(%{\"day\" => 31, \"month\" => 1, \"year\" => 2022}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~D[2022-01-31] = result\n    ~D[2022-01-31]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@date(year, month, day)\",\n    ...>   Expression.V2.Context.new(%{\"day\" => 31, \"month\" => 1, \"year\" => 2022}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-01-31\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#date/4-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.datetime_add/4","doc":"Calculates a new datetime based on the offset and unit provided.\n\nThe unit can be any of the following values:\n\n* \"Y\" for years\n* \"M\" for months\n* \"W\" for weeks\n* \"D\" for days\n* \"h\" for hours\n* \"m\" for minutes\n* \"s\" for seconds\n\nSpecifying a negative offset results in date calculations back in time.","ref":"Expression.V2.Callbacks.Standard.html#datetime_add/4"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.datetime_add/4","doc":"> Calculates a new datetime based on the offset and unit provided.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2022-08-31 00:00:00Z]` when used with the following context:\n\n```elixir\n%{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => 1, \"unit\" => \"M\"}\n```\n\n\n```\n> datetime_add(datetime, offset, unit)\n~U[2022-08-31 00:00:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(datetime, offset, unit) ...\"\n\"2022-08-31T00:00:00Z\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datetime_add(datetime, offset, unit) impact\",\n    ...>   Expression.V2.Context.new(%{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => 1, \"unit\" => \"M\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~U[2022-08-31 00:00:00Z], \" impact\"] = result\n    [\"chat for \", ~U[2022-08-31 00:00:00Z], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datetime_add(datetime, offset, unit)\",\n    ...>   Expression.V2.Context.new(%{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => 1, \"unit\" => \"M\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~U[2022-08-31 00:00:00Z] = result\n    ~U[2022-08-31 00:00:00Z]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datetime_add(datetime, offset, unit)\",\n    ...>   Expression.V2.Context.new(%{\"datetime\" => ~U[2022-07-31 00:00:00Z], \"offset\" => 1, \"unit\" => \"M\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-08-31T00:00:00Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datetime_add/4-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.datetime_add/4","doc":"> Leap year handling in a leap year.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2020-02-29 00:00:00.000000Z]`.\n\n```\n> datetime_add(date(2020, 02, 28), 1, \"D\")\n~U[2020-02-29 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2020, 02, 28), 1, \"D\") ...\"\n\"2020-02-29T00:00:00.000000Z\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datetime_add(date(2020, 02, 28), 1, \\\"D\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~U[2020-02-29 00:00:00.000000Z], \" impact\"] = result\n    [\"chat for \", ~U[2020-02-29 00:00:00.000000Z], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datetime_add(date(2020, 02, 28), 1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~U[2020-02-29 00:00:00.000000Z] = result\n    ~U[2020-02-29 00:00:00.000000Z]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datetime_add(date(2020, 02, 28), 1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2020-02-29T00:00:00.000000Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datetime_add/4-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.datetime_add/4","doc":"> Leap year handling outside of a leap year.\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2021-03-01 00:00:00.000000Z]`.\n\n```\n> datetime_add(date(2021, 02, 28), 1, \"D\")\n~U[2021-03-01 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2021, 02, 28), 1, \"D\") ...\"\n\"2021-03-01T00:00:00.000000Z\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datetime_add(date(2021, 02, 28), 1, \\\"D\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~U[2021-03-01 00:00:00.000000Z], \" impact\"] = result\n    [\"chat for \", ~U[2021-03-01 00:00:00.000000Z], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datetime_add(date(2021, 02, 28), 1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~U[2021-03-01 00:00:00.000000Z] = result\n    ~U[2021-03-01 00:00:00.000000Z]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datetime_add(date(2021, 02, 28), 1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2021-03-01T00:00:00.000000Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datetime_add/4-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.datetime_add/4","doc":"> Negative offsets\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2020-02-28 00:00:00.000000Z]`.\n\n```\n> datetime_add(date(2020, 02, 29), -1, \"D\")\n~U[2020-02-28 00:00:00.000000Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datetime_add(date(2020, 02, 29), -1, \"D\") ...\"\n\"2020-02-28T00:00:00.000000Z\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datetime_add(date(2020, 02, 29), -1, \\\"D\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~U[2020-02-28 00:00:00.000000Z], \" impact\"] = result\n    [\"chat for \", ~U[2020-02-28 00:00:00.000000Z], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datetime_add(date(2020, 02, 29), -1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~U[2020-02-28 00:00:00.000000Z] = result\n    ~U[2020-02-28 00:00:00.000000Z]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datetime_add(date(2020, 02, 29), -1, \\\"D\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2020-02-28T00:00:00.000000Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datetime_add/4-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.datevalue/3","doc":"Converts date stored in text to an actual date object and\nformats it using `strftime` formatting.\n\nIt will fallback to \"%Y-%m-%d %H:%M:%S\" if no formatting is supplied","ref":"Expression.V2.Callbacks.Standard.html#datevalue/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.datevalue/3","doc":"> Convert a date from a piece of text to a formatted date string\n\n\nWhen used in the following Stack expression it returns a complex **String** type of default value:\n```elixir\n\"2022-01-01 00:00:00\"\n```\nwith the following fields:\n\n* *date* of type **Date**\n.\n\n```\n> datevalue(\"2022-01-01\")\n%{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(\"2022-01-01\") ...\"\n\"2022-01-01 00:00:00\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datevalue(\\\"2022-01-01\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01]}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01]}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datevalue(\\\"2022-01-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01]} = result\n    %{\"__value__\" => \"2022-01-01 00:00:00\", \"date\" => ~D[2022-01-01]}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datevalue(\\\"2022-01-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-01-01 00:00:00\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datevalue/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.datevalue/3","doc":"> Convert a date from a piece of text and read the date field\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `~D[2022-01-02]`.\n\n```\n> datevalue(\"2022-01-02\").date\n~D[2022-01-02]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(\"2022-01-02\").date ...\"\n\"2022-01-02\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datevalue(\\\"2022-01-02\\\").date impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~D[2022-01-02], \" impact\"] = result\n    [\"chat for \", ~D[2022-01-02], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datevalue(\\\"2022-01-02\\\").date\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~D[2022-01-02] = result\n    ~D[2022-01-02]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datevalue(\\\"2022-01-02\\\").date\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-01-02\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datevalue/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.datevalue/3","doc":"> Convert a date value and read the date field\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `~D[2022-01-03]`.\n\n```\n> datevalue(date(2022, 1, 3)).date\n~D[2022-01-03]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(date(2022, 1, 3)).date ...\"\n\"2022-01-03\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @datevalue(date(2022, 1, 3)).date impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~D[2022-01-03], \" impact\"] = result\n    [\"chat for \", ~D[2022-01-03], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"datevalue(date(2022, 1, 3)).date\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~D[2022-01-03] = result\n    ~D[2022-01-03]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@datevalue(date(2022, 1, 3)).date\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-01-03\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#datevalue/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.day/2","doc":"Returns only the day of the month of a date (1 to 31)","ref":"Expression.V2.Callbacks.Standard.html#day/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.day/2","doc":"> Getting today's day of the month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `10`.\n\n```\n> day(date(2022, 9, 10))\n10\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @day(date(2022, 9, 10)) ...\"\n\"10\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @day(date(2022, 9, 10)) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 10, \" impact\"] = result\n    [\"chat for \", 10, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"day(date(2022, 9, 10))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 10 = result\n    10\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@day(date(2022, 9, 10))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"10\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#day/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.day/2","doc":"> Getting today's day of the month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `16`.\n\n```\n> day(now())\n16\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @day(now()) ...\"\n\"16\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @day(now()) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 16, \" impact\"] = result\n    [\"chat for \", 16, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"day(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 16 = result\n    16\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@day(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"16\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#day/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.delete/3","doc":"Deletes an element from a map by the given key.","ref":"Expression.V2.Callbacks.Standard.html#delete/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.delete/3","doc":"When used in the following Stack expression it returns a  value of type **Map**: `%{\"age\" => 32}` when used with the following context:\n\n```elixir\n%{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}}\n```\n\n\n```\n> delete(patient, \"gender\")\n%{\"age\" => 32}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @delete(patient, \"gender\") ...\"\n\"%{\"age\" => 32}\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @delete(patient, \\\"gender\\\") impact\",\n    ...>   Expression.V2.Context.new(%{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"age\" => 32}, \" impact\"] = result\n    [\"chat for \", %{\"age\" => 32}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"delete(patient, \\\"gender\\\")\",\n    ...>   Expression.V2.Context.new(%{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"age\" => 32} = result\n    %{\"age\" => 32}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@delete(patient, \\\"gender\\\")\",\n    ...>   Expression.V2.Context.new(%{\"patient\" => %{\"age\" => 32, \"gender\" => \"?\"}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"%{\\\"age\\\" => 32}\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#delete/3-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.edate/3","doc":"Moves a date by the given number of months","ref":"Expression.V2.Callbacks.Standard.html#edate/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.edate/3","doc":"> Move the date in a date object by 1 month\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2022-02-01 00:00:00Z]` when used with the following context:\n\n```elixir\n%{\"right_now\" => ~U[2022-01-01 00:00:00Z]}\n```\n\n\n```\n> edate(right_now, 1)\n~U[2022-02-01 00:00:00Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @edate(right_now, 1) ...\"\n\"2022-02-01T00:00:00Z\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @edate(right_now, 1) impact\",\n    ...>   Expression.V2.Context.new(%{\"right_now\" => ~U[2022-01-01 00:00:00Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~U[2022-02-01 00:00:00Z], \" impact\"] = result\n    [\"chat for \", ~U[2022-02-01 00:00:00Z], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"edate(right_now, 1)\",\n    ...>   Expression.V2.Context.new(%{\"right_now\" => ~U[2022-01-01 00:00:00Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~U[2022-02-01 00:00:00Z] = result\n    ~U[2022-02-01 00:00:00Z]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@edate(right_now, 1)\",\n    ...>   Expression.V2.Context.new(%{\"right_now\" => ~U[2022-01-01 00:00:00Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-02-01T00:00:00Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#edate/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.edate/3","doc":"> Move the date store in a piece of text by 1 month\n\n\nWhen used in the following Stack expression it returns a  value of type **Date**: `~D[2022-11-10]`.\n\n```\n> edate(\"2022-10-10\", 1)\n~D[2022-11-10]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @edate(\"2022-10-10\", 1) ...\"\n\"2022-11-10\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @edate(\\\"2022-10-10\\\", 1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~D[2022-11-10], \" impact\"] = result\n    [\"chat for \", ~D[2022-11-10], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"edate(\\\"2022-10-10\\\", 1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~D[2022-11-10] = result\n    ~D[2022-11-10]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@edate(\\\"2022-10-10\\\", 1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-11-10\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#edate/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.expression_docs/0","doc":"Return a list of all functions annotated with @expression_docs","ref":"Expression.V2.Callbacks.Standard.html#expression_docs/0"},{"type":"function","title":"Expression.V2.Callbacks.Standard.first_word/2","doc":"Returns the first word in the given text - equivalent to WORD(text, 1)","ref":"Expression.V2.Callbacks.Standard.html#first_word/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foo\"`.\n\n```\n> first_word(\"foo bar baz\")\n\"foo\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @first_word(\"foo bar baz\") ...\"\n\"foo\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @first_word(\\\"foo bar baz\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"foo\", \" impact\"] = result\n    [\"chat for \", \"foo\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"first_word(\\\"foo bar baz\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"foo\" = result\n    \"foo\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@first_word(\\\"foo bar baz\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"foo\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#first_word/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.fixed/4","doc":"Formats the given number in decimal format using a period and commas\n\n```\n> You have @fixed(contact.balance, 2) in your account\n\"You have 4.21 in your account\"\n```","ref":"Expression.V2.Callbacks.Standard.html#fixed/4"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.fixed/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"4.21\"`.\n\n```\n> fixed(4.209922, 2, false)\n\"4.21\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(4.209922, 2, false) ...\"\n\"4.21\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @fixed(4.209922, 2, false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"4.21\", \" impact\"] = result\n    [\"chat for \", \"4.21\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"fixed(4.209922, 2, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"4.21\" = result\n    \"4.21\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@fixed(4.209922, 2, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"4.21\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#fixed/4-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.fixed/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"4,000.4242\"`.\n\n```\n> fixed(4000.424242, 4, true)\n\"4,000.4242\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(4000.424242, 4, true) ...\"\n\"4,000.4242\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @fixed(4000.424242, 4, true) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"4,000.4242\", \" impact\"] = result\n    [\"chat for \", \"4,000.4242\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"fixed(4000.424242, 4, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"4,000.4242\" = result\n    \"4,000.4242\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@fixed(4000.424242, 4, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"4,000.4242\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#fixed/4-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.fixed/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"3.80\"`.\n\n```\n> fixed(3.7979, 2, false)\n\"3.80\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(3.7979, 2, false) ...\"\n\"3.80\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @fixed(3.7979, 2, false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"3.80\", \" impact\"] = result\n    [\"chat for \", \"3.80\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"fixed(3.7979, 2, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"3.80\" = result\n    \"3.80\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@fixed(3.7979, 2, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3.80\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#fixed/4-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.fixed/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"3.80\"`.\n\n```\n> fixed(3.7979, 2)\n\"3.80\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @fixed(3.7979, 2) ...\"\n\"3.80\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @fixed(3.7979, 2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"3.80\", \" impact\"] = result\n    [\"chat for \", \"3.80\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"fixed(3.7979, 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"3.80\" = result\n    \"3.80\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@fixed(3.7979, 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3.80\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#fixed/4-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_all_words/3","doc":"Tests whether all the words are contained in text\n\nThe words can be in any order and may appear more than once.","ref":"Expression.V2.Callbacks.Standard.html#has_all_words/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_all_words/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_all_words(\"the quick brown FOX\", \"the fox\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_all_words(\"the quick brown FOX\", \"the fox\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_all_words(\\\"the quick brown FOX\\\", \\\"the fox\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_all_words(\\\"the quick brown FOX\\\", \\\"the fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_all_words(\\\"the quick brown FOX\\\", \\\"the fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_all_words/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_all_words/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_all_words(\"the quick brown FOX\", \"red fox\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_all_words(\"the quick brown FOX\", \"red fox\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_all_words(\\\"the quick brown FOX\\\", \\\"red fox\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_all_words(\\\"the quick brown FOX\\\", \\\"red fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_all_words(\\\"the quick brown FOX\\\", \\\"red fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_all_words/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_any_word/3","doc":"Tests whether any of the words are contained in the text\n\nOnly one of the words needs to match and it may appear more than once.","ref":"Expression.V2.Callbacks.Standard.html#has_any_word/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_any_word/3","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```elixir\ntrue\n```\nwith the following fields:\n\n* *match* of type **String**\n.\n\n```\n> has_any_word(\"The Quick Brown Fox\", \"fox quick\")\n%{\"__value__\" => true, \"match\" => \"Quick Fox\"}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_any_word(\"The Quick Brown Fox\", \"fox quick\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_any_word(\\\"The Quick Brown Fox\\\", \\\"fox quick\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => true, \"match\" => \"Quick Fox\"}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => true, \"match\" => \"Quick Fox\"}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_any_word(\\\"The Quick Brown Fox\\\", \\\"fox quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => true, \"match\" => \"Quick Fox\"} = result\n    %{\"__value__\" => true, \"match\" => \"Quick Fox\"}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_any_word(\\\"The Quick Brown Fox\\\", \\\"fox quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_any_word/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_any_word/3","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```elixir\nfalse\n```\nwith the following fields:\n\n* *match* of type **Null**\n.\n\n```\n> has_any_word(\"The Quick Brown Fox\", \"yellow\")\n%{\"__value__\" => false, \"match\" => nil}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_any_word(\"The Quick Brown Fox\", \"yellow\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_any_word(\\\"The Quick Brown Fox\\\", \\\"yellow\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => false, \"match\" => nil}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => false, \"match\" => nil}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_any_word(\\\"The Quick Brown Fox\\\", \\\"yellow\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => false, \"match\" => nil} = result\n    %{\"__value__\" => false, \"match\" => nil}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_any_word(\\\"The Quick Brown Fox\\\", \\\"yellow\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_any_word/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_beginning/3","doc":"Tests whether text starts with beginning\n\nBoth text values are trimmed of surrounding whitespace, but otherwise matching is\nstrict without any tokenization.","ref":"Expression.V2.Callbacks.Standard.html#has_beginning/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_beginning(\"The Quick Brown\", \"the quick\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"the quick\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_beginning(\\\"The Quick Brown\\\", \\\"the quick\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"the quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"the quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_beginning/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_beginning(\"The Quick Brown\", \"the    quick\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"the    quick\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_beginning(\\\"The Quick Brown\\\", \\\"the    quick\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"the    quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"the    quick\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_beginning/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_beginning/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_beginning(\"The Quick Brown\", \"quick brown\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_beginning(\"The Quick Brown\", \"quick brown\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_beginning(\\\"The Quick Brown\\\", \\\"quick brown\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_beginning(\\\"The Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_beginning(\\\"The Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_beginning/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_date/2","doc":"Tests whether `expression` contains a date formatted according to our environment\n\nThis is very naively implemented with a regular expression.","ref":"Expression.V2.Callbacks.Standard.html#has_date/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date(\"the date is 15/01/2017\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"the date is 15/01/2017\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date(\\\"the date is 15/01/2017\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date(\\\"the date is 15/01/2017\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date(\\\"the date is 15/01/2017\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_date/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date(\"there is no date here, just a year 2017\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date(\"there is no date here, just a year 2017\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date(\\\"there is no date here, just a year 2017\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date(\\\"there is no date here, just a year 2017\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date(\\\"there is no date here, just a year 2017\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_date_eq/3","doc":"Tests whether `expression` is a date equal to `date_string`","ref":"Expression.V2.Callbacks.Standard.html#has_date_eq/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_date_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_eq(\"the date is 15/01/2017\", \"2017-01-15\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_eq(\"the date is 15/01/2017\", \"2017-01-15\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_eq(\\\"the date is 15/01/2017\\\", \\\"2017-01-15\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_eq(\\\"the date is 15/01/2017\\\", \\\"2017-01-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_eq(\\\"the date is 15/01/2017\\\", \\\"2017-01-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_eq/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_date_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_eq(\"there is no date here, just a year 2017\", \"2017-01-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_eq(\"there is no date here, just a year 2017\", \"2017-01-15\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_eq(\\\"there is no date here, just a year 2017\\\", \\\"2017-01-15\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_eq(\\\"there is no date here, just a year 2017\\\", \\\"2017-01-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_eq(\\\"there is no date here, just a year 2017\\\", \\\"2017-01-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_eq/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_date_gt/3","doc":"Tests whether `expression` is a date after the date `date_string`","ref":"Expression.V2.Callbacks.Standard.html#has_date_gt/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_date_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_gt(\"the date is 15/01/2017\", \"2017-01-01\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_gt(\"the date is 15/01/2017\", \"2017-01-01\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-01-01\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-01-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-01-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_gt/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_date_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_gt(\"the date is 15/01/2017\", \"2017-03-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_gt(\"the date is 15/01/2017\", \"2017-03-15\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-03-15\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-03-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_gt(\\\"the date is 15/01/2017\\\", \\\"2017-03-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_gt/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_date_lt/3","doc":"Tests whether `expression` contains a date before the date `date_string`","ref":"Expression.V2.Callbacks.Standard.html#has_date_lt/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_date_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_date_lt(\"the date is 15/01/2017\", \"2017-06-01\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_lt(\"the date is 15/01/2017\", \"2017-06-01\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_lt(\\\"the date is 15/01/2017\\\", \\\"2017-06-01\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_lt(\\\"the date is 15/01/2017\\\", \\\"2017-06-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_lt(\\\"the date is 15/01/2017\\\", \\\"2017-06-01\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_lt/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_date_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_date_lt(\"the date is 15/01/2021\", \"2017-03-15\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_date_lt(\"the date is 15/01/2021\", \"2017-03-15\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_date_lt(\\\"the date is 15/01/2021\\\", \\\"2017-03-15\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_date_lt(\\\"the date is 15/01/2021\\\", \\\"2017-03-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_date_lt(\\\"the date is 15/01/2021\\\", \\\"2017-03-15\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_date_lt/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_email/2","doc":"Tests whether an email is contained in text","ref":"Expression.V2.Callbacks.Standard.html#has_email/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_email/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_email(\"my email is foo1@bar.com, please respond\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_email(\"my email is foo1@bar.com, please respond\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_email(\\\"my email is foo1@bar.com, please respond\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_email(\\\"my email is foo1@bar.com, please respond\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_email(\\\"my email is foo1@bar.com, please respond\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_email/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_email/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_email(\"i'm not sharing my email\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_email(\"i'm not sharing my email\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_email(\\\"i'm not sharing my email\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_email(\\\"i'm not sharing my email\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_email(\\\"i'm not sharing my email\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_email/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_group/3","doc":"Returns whether the contact is part of group with the passed in UUID","ref":"Expression.V2.Callbacks.Standard.html#has_group/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_group/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}\n```\n\n\n```\n> has_group(contact.groups, \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_group(contact.groups, \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_group(contact.groups, \\\"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\\\") impact\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_group(contact.groups, \\\"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\\\")\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_group(contact.groups, \\\"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\\\")\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_group/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_group/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false` when used with the following context:\n\n```elixir\n%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}\n```\n\n\n```\n> has_group(contact.groups, \"00000000-0000-0000-0000-000000000000\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_group(contact.groups, \"00000000-0000-0000-0000-000000000000\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_group(contact.groups, \\\"00000000-0000-0000-0000-000000000000\\\") impact\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_group(contact.groups, \\\"00000000-0000-0000-0000-000000000000\\\")\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_group(contact.groups, \\\"00000000-0000-0000-0000-000000000000\\\")\",\n    ...>   Expression.V2.Context.new(%{\"contact\" => %{\"groups\" => [%{\"uuid\" => \"b7cf0d83-f1c9-411c-96fd-c511a4cfa86d\"}]}}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_group/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number/2","doc":"Tests whether `expression` contains a number","ref":"Expression.V2.Callbacks.Standard.html#has_number/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number(\"the number is 42 and 5\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"the number is 42 and 5\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number(\\\"the number is 42 and 5\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number(\\\"the number is 42 and 5\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number(\\\"the number is 42 and 5\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number(\"العدد ٤٢\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"العدد ٤٢\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number(\\\"العدد ٤٢\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number(\\\"العدد ٤٢\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number(\\\"العدد ٤٢\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number(\"٠.٥\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"٠.٥\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number(\\\"٠.٥\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number(\\\"٠.٥\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number(\\\"٠.٥\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number(\"0.6\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number(\"0.6\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number(\\\"0.6\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number(\\\"0.6\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number(\\\"0.6\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"Tests whether `expression` contains a number equal to the value","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 42\\\", 42) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 42\\\", 42.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 42\\\", \\\"42\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_eq(\"the number is 42.0\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 42.0\", \"42\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 42.0\\\", \\\"42\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 42.0\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 42.0\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"the number is 40\", \"42\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 40\", \"42\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 40\\\", \\\"42\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 40\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 40\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"the number is 40\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-6"},{"type":"function","title":"Example 7: - Expression.V2.Callbacks.Standard.has_number_eq/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_eq(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_eq(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_eq(\\\"four hundred\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_eq(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_eq(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_eq/3-example-7"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"Tests whether `expression` contains a number greater than min","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", 40)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", 40) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 42\\\", 40) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", 40)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", 40)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", 40.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", 40.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 42\\\", 40.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", 40.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", 40.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42\", \"40\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42\", \"40\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 42\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gt(\"the number is 42.0\", \"40\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 42.0\", \"40\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 42.0\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"the number is 40\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 40\", \"40\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 40\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"the number is 40\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-6"},{"type":"function","title":"Example 7: - Expression.V2.Callbacks.Standard.has_number_gt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gt(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gt(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gt(\\\"four hundred\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gt/3-example-7"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"Tests whether `expression` contains a number greater than or equal to min","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 42\\\", 42) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 42\\\", 42.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_gte(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 42\\\", \\\"42\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 42.0\", \"45\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 42.0\", \"45\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 42.0\\\", \\\"45\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 42.0\\\", \\\"45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 42.0\\\", \\\"45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 40\", \"45\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 40\", \"45\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 40\\\", \\\"45\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 40\\\", \\\"45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 40\\\", \\\"45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"the number is 40\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-6"},{"type":"function","title":"Example 7: - Expression.V2.Callbacks.Standard.has_number_gte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_gte(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_gte(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_gte(\\\"four hundred\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_gte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_gte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_gte/3-example-7"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"Tests whether `expression` contains a number less than max","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lt(\"the number is 42\", 44)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", 44) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 42\\\", 44) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", 44)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", 44)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lt(\"the number is 42\", 44.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", 44.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 42\\\", 44.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", 44.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", 44.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 42\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42\", \"40\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 42\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 42\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 42.0\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 42.0\", \"40\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 42.0\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 40\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 40\", \"40\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 40\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 40\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"the number is 40\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-6"},{"type":"function","title":"Example 7: - Expression.V2.Callbacks.Standard.has_number_lt/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lt(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lt(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lt(\\\"four hundred\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lt(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lt/3-example-7"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"Tests whether `expression` contains a number less than or equal to max","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", 42)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", 42) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"the number is 42\\\", 42) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", 42)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", 42.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", 42.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"the number is 42\\\", 42.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", 42.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_number_lte(\"the number is 42\", \"42\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42\", \"42\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"the number is 42\\\", \\\"42\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"the number is 42\\\", \\\"42\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"the number is 42.0\", \"40\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 42.0\", \"40\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"the number is 42.0\\\", \\\"40\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"the number is 42.0\\\", \\\"40\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"the number is 40\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"the number is 40\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"the number is 40\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"the number is 40\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_number_lte(\"four hundred\", \"foo\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(\"four hundred\", \"foo\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(\\\"four hundred\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(\\\"four hundred\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-6"},{"type":"function","title":"Example 7: - Expression.V2.Callbacks.Standard.has_number_lte/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true` when used with the following context:\n\n```elixir\n%{\"response\" => 3}\n```\n\n\n```\n> has_number_lte(response, 5)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_number_lte(response, 5) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_number_lte(response, 5) impact\",\n    ...>   Expression.V2.Context.new(%{\"response\" => 3}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_number_lte(response, 5)\",\n    ...>   Expression.V2.Context.new(%{\"response\" => 3}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_number_lte(response, 5)\",\n    ...>   Expression.V2.Context.new(%{\"response\" => 3}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_number_lte/3-example-7"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_only_phrase/3","doc":"Tests whether the text contains only phrase\n\nThe phrase must be the only text in the text to match","ref":"Expression.V2.Callbacks.Standard.html#has_only_phrase/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_phrase(\"Quick Brown\", \"quick brown\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"Quick Brown\", \"quick brown\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_phrase(\\\"Quick Brown\\\", \\\"quick brown\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_phrase(\\\"Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_phrase(\\\"Quick Brown\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_phrase/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_phrase(\"\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"\", \"\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_phrase(\\\"\\\", \\\"\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_phrase(\\\"\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_phrase(\\\"\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_phrase/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_only_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_only_phrase(\"The Quick Brown Fox\", \"quick brown\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_phrase(\"The Quick Brown Fox\", \"quick brown\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_phrase(\\\"The Quick Brown Fox\\\", \\\"quick brown\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_phrase(\\\"The Quick Brown Fox\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_phrase(\\\"The Quick Brown Fox\\\", \\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_phrase/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_only_text/3","doc":"Returns whether two text values are equal (case sensitive). In the case that they are, it will return the text as the match.","ref":"Expression.V2.Callbacks.Standard.html#has_only_text/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_text(\"foo\", \"foo\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"foo\", \"foo\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_text(\\\"foo\\\", \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_text(\\\"foo\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_text(\\\"foo\\\", \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_text/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_only_text(\"\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"\", \"\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_text(\\\"\\\", \\\"\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_text(\\\"\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_text(\\\"\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_text/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_only_text/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_only_text(\"foo\", \"FOO\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_only_text(\"foo\", \"FOO\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_only_text(\\\"foo\\\", \\\"FOO\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_only_text(\\\"foo\\\", \\\"FOO\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_only_text(\\\"foo\\\", \\\"FOO\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_only_text/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_pattern/3","doc":"Tests whether `expression` matches the regex pattern\n\nBoth text values are trimmed of surrounding whitespace and matching is case-insensitive.","ref":"Expression.V2.Callbacks.Standard.html#has_pattern/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_pattern/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_pattern(\"Buy cheese please\", \"buy (\\w+)\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_pattern(\"Buy cheese please\", \"buy (\\w+)\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_pattern(\\\"Buy cheese please\\\", \\\"buy (\\\\w+)\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_pattern(\\\"Buy cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_pattern(\\\"Buy cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_pattern/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_pattern/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_pattern(\"Sell cheese please\", \"buy (\\w+)\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_pattern(\"Sell cheese please\", \"buy (\\w+)\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_pattern(\\\"Sell cheese please\\\", \\\"buy (\\\\w+)\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_pattern(\\\"Sell cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_pattern(\\\"Sell cheese please\\\", \\\"buy (\\\\w+)\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_pattern/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_phone/2","doc":"Tests whether `expression` contains a phone number.\nThe optional country_code argument specifies the country to use for parsing.","ref":"Expression.V2.Callbacks.Standard.html#has_phone/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phone(\"my number is +12067799294 thanks\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is +12067799294 thanks\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phone(\\\"my number is +12067799294 thanks\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phone(\\\"my number is +12067799294 thanks\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phone(\\\"my number is +12067799294 thanks\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phone/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phone(\"my number is 2067799294 thanks\", \"US\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is 2067799294 thanks\", \"US\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phone(\\\"my number is 2067799294 thanks\\\", \\\"US\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phone(\\\"my number is 2067799294 thanks\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phone(\\\"my number is 2067799294 thanks\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phone/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phone(\"my number is 206 779 9294 thanks\", \"US\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is 206 779 9294 thanks\", \"US\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phone(\\\"my number is 206 779 9294 thanks\\\", \\\"US\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phone(\\\"my number is 206 779 9294 thanks\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phone(\\\"my number is 206 779 9294 thanks\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phone/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_phone/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_phone(\"my number is none of your business\", \"US\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phone(\"my number is none of your business\", \"US\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phone(\\\"my number is none of your business\\\", \\\"US\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phone(\\\"my number is none of your business\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phone(\\\"my number is none of your business\\\", \\\"US\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phone/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_phone/3","doc":"","ref":"Expression.V2.Callbacks.Standard.html#has_phone/3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_phrase/3","doc":"Tests whether phrase is contained in `expression`\n\nThe words in the test phrase must appear in the same order with no other words in between.","ref":"Expression.V2.Callbacks.Standard.html#has_phrase/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phrase(\"the quick brown fox\", \"brown fox\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"brown fox\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phrase(\\\"the quick brown fox\\\", \\\"brown fox\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"brown fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"brown fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phrase/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_phrase(\"the quick brown fox\", \"quick fox\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"quick fox\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phrase(\\\"the quick brown fox\\\", \\\"quick fox\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"quick fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"quick fox\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phrase/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_phrase/3","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_phrase(\"the quick brown fox\", \"\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_phrase(\"the quick brown fox\", \"\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_phrase(\\\"the quick brown fox\\\", \\\"\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_phrase(\\\"the quick brown fox\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_phrase(\\\"the quick brown fox\\\", \\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_phrase/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_text/2","doc":"Tests whether there the `expression` has any characters in it","ref":"Expression.V2.Callbacks.Standard.html#has_text/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_text(\"quick brown\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\"quick brown\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_text(\\\"quick brown\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_text(\\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_text(\\\"quick brown\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_text/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_text(\"\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\"\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_text(\\\"\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_text(\\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_text(\\\"\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_text/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_text(\" \n> \")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(\" \n\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_text(\\\" \\n\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_text(\\\" \\n\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_text(\\\" \\n\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_text/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_text/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> has_text(123)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_text(123) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_text(123) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_text(123)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_text(123)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_text/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.has_time/2","doc":"Tests whether `expression` contains a time.","ref":"Expression.V2.Callbacks.Standard.html#has_time/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```elixir\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:30\")\n%{\"__value__\" => true, \"match\" => ~T[10:30:00]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:30\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_time(\\\"the time is 10:30\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:30:00]}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:30:00]}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_time(\\\"the time is 10:30\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:30:00]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:30:00]}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_time(\\\"the time is 10:30\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_time/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```elixir\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:00 pm\")\n%{\"__value__\" => true, \"match\" => ~T[10:00:00]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:00 pm\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_time(\\\"the time is 10:00 pm\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:00:00]}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:00:00]}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_time(\\\"the time is 10:00 pm\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:00:00]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:00:00]}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_time(\\\"the time is 10:00 pm\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_time/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a complex **Boolean** type of default value:\n```elixir\ntrue\n```\nwith the following fields:\n\n* *match* of type **Time**\n.\n\n```\n> has_time(\"the time is 10:30:45\")\n%{\"__value__\" => true, \"match\" => ~T[10:30:45]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"the time is 10:30:45\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_time(\\\"the time is 10:30:45\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:30:45]}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => true, \"match\" => ~T[10:30:45]}, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_time(\\\"the time is 10:30:45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert %{\"__value__\" => true, \"match\" => ~T[10:30:45]} = result\n    %{\"__value__\" => true, \"match\" => ~T[10:30:45]}\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_time(\\\"the time is 10:30:45\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_time/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.has_time/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> has_time(\"there is no time here, just the number 25\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @has_time(\"there is no time here, just the number 25\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @has_time(\\\"there is no time here, just the number 25\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"has_time(\\\"there is no time here, just the number 25\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@has_time(\\\"there is no time here, just the number 25\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#has_time/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.hour/2","doc":"Returns only the hour of a datetime (0 to 23)","ref":"Expression.V2.Callbacks.Standard.html#hour/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.hour/2","doc":"> Get the current hour\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `12`.\n\n```\n> hour(now())\n12\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @hour(now()) ...\"\n\"12\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @hour(now()) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 12, \" impact\"] = result\n    [\"chat for \", 12, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"hour(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 12 = result\n    12\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@hour(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"12\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#hour/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.isbool/2","doc":"Returns `true` if the argument is a boolean.","ref":"Expression.V2.Callbacks.Standard.html#isbool/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isbool(true)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(true) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(true) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isbool(false)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(false) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(1)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(1) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(0)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(0) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-4"},{"type":"function","title":"Example 5: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(\"true\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(\"true\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(\\\"true\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(\\\"true\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(\\\"true\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-5"},{"type":"function","title":"Example 6: - Expression.V2.Callbacks.Standard.isbool/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isbool(\"false\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isbool(\"false\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isbool(\\\"false\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isbool(\\\"false\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isbool(\\\"false\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isbool/2-example-6"},{"type":"function","title":"Expression.V2.Callbacks.Standard.isnumber/2","doc":"Returns `true` if the argument is a number.","ref":"Expression.V2.Callbacks.Standard.html#isnumber/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(1)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(1) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isnumber(1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isnumber(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isnumber(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isnumber/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(1.0)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(1.0) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isnumber(1.0) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isnumber(1.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isnumber(1.0)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isnumber/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isnumber(\"1.0\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(\"1.0\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isnumber(\\\"1.0\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isnumber(\\\"1.0\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isnumber(\\\"1.0\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isnumber/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.isnumber/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isnumber(\"a\")\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isnumber(\"a\") ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isnumber(\\\"a\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isnumber(\\\"a\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isnumber(\\\"a\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isnumber/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.isstring/2","doc":"Returns `true` if the argument is a string.","ref":"Expression.V2.Callbacks.Standard.html#isstring/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> isstring(\"hello\")\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(\"hello\") ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isstring(\\\"hello\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isstring(\\\"hello\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isstring(\\\"hello\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isstring/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isstring(false)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(false) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isstring(false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isstring(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isstring(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isstring/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.isstring/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> isstring(1)\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @isstring(1) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @isstring(1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"isstring(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@isstring(1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#isstring/2-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.left/3","doc":"Returns the first characters in a text string. This is Unicode safe.","ref":"Expression.V2.Callbacks.Standard.html#left/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.left/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foob\"`.\n\n```\n> left(\"foobar\", 4)\n\"foob\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @left(\"foobar\", 4) ...\"\n\"foob\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @left(\\\"foobar\\\", 4) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"foob\", \" impact\"] = result\n    [\"chat for \", \"foob\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"left(\\\"foobar\\\", 4)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"foob\" = result\n    \"foob\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@left(\\\"foobar\\\", 4)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"foob\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#left/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.left/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"Умерла Мадлен Олбрай\"`.\n\n```\n> left(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20)\n\"Умерла Мадлен Олбрай\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @left(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20) ...\"\n\"Умерла Мадлен Олбрай\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @left(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"Умерла Мадлен Олбрай\", \" impact\"] = result\n    [\"chat for \", \"Умерла Мадлен Олбрай\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"left(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"Умерла Мадлен Олбрай\" = result\n    \"Умерла Мадлен Олбрай\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@left(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"Умерла Мадлен Олбрай\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#left/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.len/2","doc":"Returns the number of characters in a text string","ref":"Expression.V2.Callbacks.Standard.html#len/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.len/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> len(\"foo\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @len(\"foo\") ...\"\n\"3\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @len(\\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 3, \" impact\"] = result\n    [\"chat for \", 3, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"len(\\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 3 = result\n    3\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@len(\\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#len/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.len/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> len(\"zoë\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @len(\"zoë\") ...\"\n\"3\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @len(\\\"zoë\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 3, \" impact\"] = result\n    [\"chat for \", 3, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"len(\\\"zoë\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 3 = result\n    3\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@len(\\\"zoë\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#len/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.lower/2","doc":"Converts a text string to lowercase","ref":"Expression.V2.Callbacks.Standard.html#lower/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.lower/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"foo bar\"`.\n\n```\n> lower(\"Foo Bar\")\n\"foo bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @lower(\"Foo Bar\") ...\"\n\"foo bar\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @lower(\\\"Foo Bar\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"foo bar\", \" impact\"] = result\n    [\"chat for \", \"foo bar\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"lower(\\\"Foo Bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"foo bar\" = result\n    \"foo bar\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@lower(\\\"Foo Bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"foo bar\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#lower/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.map/3","doc":"map over a list of items and apply the mapper function to every item, returning\nthe result.","ref":"Expression.V2.Callbacks.Standard.html#map/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.map/3","doc":"> Map over the range of numbers, create a date in January for every number\n\n\nWhen used in the following Stack expression it returns a  value of type **List with values Date, Date, Date**: `[~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]]`.\n\n```\n> map(1..3, &date(2022, 1, &1))\n[~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @map(1..3, &date(2022, 1, &1)) ...\"\n\"2022-01-012022-01-022022-01-03\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @map(1..3, &date(2022, 1, &1)) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]], \" impact\"] = result\n    [\"chat for \", [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"map(1..3, &date(2022, 1, &1))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]] = result\n    [~D[2022-01-01], ~D[2022-01-02], ~D[2022-01-03]]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@map(1..3, &date(2022, 1, &1))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2022-01-012022-01-022022-01-03\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#map/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.map/3","doc":"> Map over the range of numbers, multiple each by itself and return the result\n\n\nWhen used in the following Stack expression it returns a  value of type **List with values Integer, Integer, Integer**: `[1, 4, 9]`.\n\n```\n> map(1..3, &(&1 * &1))\n[1, 4, 9]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @map(1..3, &(&1 * &1)) ...\"\n\"149\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @map(1..3, &(&1 * &1)) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", [1, 4, 9], \" impact\"] = result\n    [\"chat for \", [1, 4, 9], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"map(1..3, &(&1 * &1))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert [1, 4, 9] = result\n    [1, 4, 9]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@map(1..3, &(&1 * &1))\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"149\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#map/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.max_vargs/2","doc":"Returns the maximum value of all arguments","ref":"Expression.V2.Callbacks.Standard.html#max_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.max_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> max(1, 2, 3)\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @max(1, 2, 3) ...\"\n\"3\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @max(1, 2, 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 3, \" impact\"] = result\n    [\"chat for \", 3, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"max(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 3 = result\n    3\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@max(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#max_vargs/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.min_vargs/2","doc":"Returns the minimum value of all arguments","ref":"Expression.V2.Callbacks.Standard.html#min_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.min_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> min(1, 2, 3)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @min(1, 2, 3) ...\"\n\"1\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @min(1, 2, 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 1, \" impact\"] = result\n    [\"chat for \", 1, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"min(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 1 = result\n    1\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@min(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#min_vargs/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.minute/2","doc":"Returns only the minute of a datetime (0 to 59)","ref":"Expression.V2.Callbacks.Standard.html#minute/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.minute/2","doc":"> Get the current minute\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `53`.\n\n```\n> minute(now())\n53\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @minute(now()) ...\"\n\"53\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @minute(now()) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 53, \" impact\"] = result\n    [\"chat for \", 53, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"minute(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 53 = result\n    53\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@minute(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"53\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#minute/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.month/2","doc":"Returns only the month of a date (1 to 12)","ref":"Expression.V2.Callbacks.Standard.html#month/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.month/2","doc":"> Get the current month\n\n\nWhen used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> month(now())\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @month(now()) ...\"\n\"1\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @month(now()) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 1, \" impact\"] = result\n    [\"chat for \", 1, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"month(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 1 = result\n    1\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@month(now())\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#month/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.not_/2","doc":"Returns `false` if the argument supplied evaluates to truth-y","ref":"Expression.V2.Callbacks.Standard.html#not_/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.not_/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> not(false)\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @not(false) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @not(false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"not(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@not(false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#not_/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.now/1","doc":"Returns the current date time as UTC\n\n```\nIt is currently @NOW()\n```","ref":"Expression.V2.Callbacks.Standard.html#now/1"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.now/1","doc":"> return the current timestamp as a DateTime value\n\n\nWhen used in the following Stack expression it returns a  value of type **DateTime**: `~U[2024-01-16 12:53:57.258232Z]`.\n\n```\n> now()\n~U[2024-01-16 12:53:57.258232Z]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @now() ...\"\n\"2024-01-16T12:53:57.258232Z\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @now() impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", ~U[2024-01-16 12:53:57.258232Z], \" impact\"] = result\n    [\"chat for \", ~U[2024-01-16 12:53:57.258232Z], \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"now()\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert ~U[2024-01-16 12:53:57.258232Z] = result\n    ~U[2024-01-16 12:53:57.258232Z]\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@now()\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2024-01-16T12:53:57.258232Z\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#now/1-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.now/1","doc":"> return the current datetime and format it using `datevalue`\n\n\nWhen used in the following Stack expression it returns a complex **String** type of default value:\n```elixir\n\"2024-01-16\"\n```\nwith the following fields:\n\n* *date* of type **DateTime**\n.\n\n```\n> datevalue(now(), \"%Y-%m-%d\")\n%{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @datevalue(now(), \"%Y-%m-%d\") ...\"\n\"2024-01-16\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @datevalue(now(), \\\"%Y-%m-%d\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}, \" impact\"] = result\n    [\"chat for \", %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"datevalue(now(), \\\"%Y-%m-%d\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]} = result\n    %{\"__value__\" => \"2024-01-16\", \"date\" => ~U[2024-01-16 12:53:57.311266Z]}\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@datevalue(now(), \\\"%Y-%m-%d\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2024-01-16\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#now/1-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.or_vargs/2","doc":"Returns `true` if any argument is `true`.\nReturns the first truthy value found or otherwise false.\n\nAccepts any amount of arguments for testing truthiness.","ref":"Expression.V2.Callbacks.Standard.html#or_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.or_vargs/2","doc":"> Return true if any of the values are true\n\n\nWhen used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> true or false\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(true, false) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @or(true, false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"or(true, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@or(true, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#or_vargs/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.or_vargs/2","doc":"> Return the first value that is truthy\n\n\nWhen used in the following Stack expression it returns a  value of type **String**: `\"foo\"`.\n\n```\n> false or \"foo\"\n\"foo\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(false, \"foo\") ...\"\n\"foo\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @or(false, \\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"foo\", \" impact\"] = result\n    [\"chat for \", \"foo\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"or(false, \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"foo\" = result\n    \"foo\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@or(false, \\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"foo\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#or_vargs/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.or_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `true`.\n\n```\n> true or true\ntrue\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(true, true) ...\"\n\"true\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @or(true, true) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", true, \" impact\"] = result\n    [\"chat for \", true, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"or(true, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert true = result\n    true\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@or(true, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"true\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#or_vargs/2-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.or_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Boolean**: `false`.\n\n```\n> false or false\nfalse\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @or(false, false) ...\"\n\"false\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @or(false, false) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", false, \" impact\"] = result\n    [\"chat for \", false, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"or(false, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> refute result\n    false\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@or(false, false)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"false\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#or_vargs/2-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.parse_float/1","doc":"","ref":"Expression.V2.Callbacks.Standard.html#parse_float/1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.percent/2","doc":"Formats a number as a percentage","ref":"Expression.V2.Callbacks.Standard.html#percent/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"`.\n\n```\n> percent(2/10)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(2/10) ...\"\n\"20%\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @percent(2/10) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"20%\", \" impact\"] = result\n    [\"chat for \", \"20%\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"percent(2/10)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@percent(2/10)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#percent/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"`.\n\n```\n> percent(0.2)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(0.2) ...\"\n\"20%\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @percent(0.2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"20%\", \" impact\"] = result\n    [\"chat for \", \"20%\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"percent(0.2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@percent(0.2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#percent/2-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.percent/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"20%\"` when used with the following context:\n\n```elixir\n%{\"d\" => \"0.2\"}\n```\n\n\n```\n> percent(d)\n\"20%\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @percent(d) ...\"\n\"20%\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @percent(d) impact\",\n    ...>   Expression.V2.Context.new(%{\"d\" => \"0.2\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"20%\", \" impact\"] = result\n    [\"chat for \", \"20%\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"percent(d)\",\n    ...>   Expression.V2.Context.new(%{\"d\" => \"0.2\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"20%\" = result\n    \"20%\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@percent(d)\",\n    ...>   Expression.V2.Context.new(%{\"d\" => \"0.2\"}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"20%\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#percent/2-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.power/3","doc":"Returns the result of a number raised to a power - equivalent to the ^ operator","ref":"Expression.V2.Callbacks.Standard.html#power/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.power/3","doc":"When used in the following Stack expression it returns a  value of type **Float**: `8.0`.\n\n```\n> power(2, 3)\n8.0\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @power(2, 3) ...\"\n\"8.0\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @power(2, 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 8.0, \" impact\"] = result\n    [\"chat for \", 8.0, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"power(2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 8.0 = result\n    8.0\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@power(2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"8.0\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#power/3-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.proper/2","doc":"Capitalizes the first letter of every word in a text string","ref":"Expression.V2.Callbacks.Standard.html#proper/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.proper/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"Foo Bar\"`.\n\n```\n> proper(\"foo bar\")\n\"Foo Bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @proper(\"foo bar\") ...\"\n\"Foo Bar\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @proper(\\\"foo bar\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"Foo Bar\", \" impact\"] = result\n    [\"chat for \", \"Foo Bar\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"proper(\\\"foo bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"Foo Bar\" = result\n    \"Foo Bar\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@proper(\\\"foo bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"Foo Bar\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#proper/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.read_digits/2","doc":"Formats digits in text for reading in TTS","ref":"Expression.V2.Callbacks.Standard.html#read_digits/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.read_digits/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"plus two seven one\"`.\n\n```\n> read_digits(\"+271\")\n\"plus two seven one\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @read_digits(\"+271\") ...\"\n\"plus two seven one\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @read_digits(\\\"+271\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"plus two seven one\", \" impact\"] = result\n    [\"chat for \", \"plus two seven one\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"read_digits(\\\"+271\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"plus two seven one\" = result\n    \"plus two seven one\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@read_digits(\\\"+271\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"plus two seven one\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#read_digits/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.rem/3","doc":"Return the division remainder of two integers.","ref":"Expression.V2.Callbacks.Standard.html#rem/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.rem/3","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `0`.\n\n```\n> rem(4, 2)\n0\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rem(4, 2) ...\"\n\"0\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @rem(4, 2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 0, \" impact\"] = result\n    [\"chat for \", 0, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"rem(4, 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 0 = result\n    0\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@rem(4, 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"0\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#rem/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.rem/3","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1`.\n\n```\n> rem(85, 3)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rem(85, 3) ...\"\n\"1\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @rem(85, 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 1, \" impact\"] = result\n    [\"chat for \", 1, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"rem(85, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 1 = result\n    1\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@rem(85, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#rem/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.remove_first_word/2","doc":"Removes the first word from the given text. The remaining text will be unchanged","ref":"Expression.V2.Callbacks.Standard.html#remove_first_word/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.remove_first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"bar\"`.\n\n```\n> remove_first_word(\"foo bar\")\n\"bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @remove_first_word(\"foo bar\") ...\"\n\"bar\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @remove_first_word(\\\"foo bar\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"bar\", \" impact\"] = result\n    [\"chat for \", \"bar\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"remove_first_word(\\\"foo bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"bar\" = result\n    \"bar\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@remove_first_word(\\\"foo bar\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"bar\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#remove_first_word/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.remove_first_word/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"bar\"`.\n\n```\n> remove_first_word(\"foo-bar\", \"-\")\n\"bar\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @remove_first_word(\"foo-bar\", \"-\") ...\"\n\"bar\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @remove_first_word(\\\"foo-bar\\\", \\\"-\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"bar\", \" impact\"] = result\n    [\"chat for \", \"bar\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"remove_first_word(\\\"foo-bar\\\", \\\"-\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"bar\" = result\n    \"bar\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@remove_first_word(\\\"foo-bar\\\", \\\"-\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"bar\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#remove_first_word/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.remove_first_word/3","doc":"","ref":"Expression.V2.Callbacks.Standard.html#remove_first_word/3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.rept/3","doc":"Repeats text a given number of times","ref":"Expression.V2.Callbacks.Standard.html#rept/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.rept/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"**********\"`.\n\n```\n> rept(\"*\", 10)\n\"**********\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @rept(\"*\", 10) ...\"\n\"**********\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @rept(\\\"*\\\", 10) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"**********\", \" impact\"] = result\n    [\"chat for \", \"**********\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"rept(\\\"*\\\", 10)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"**********\" = result\n    \"**********\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@rept(\\\"*\\\", 10)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"**********\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#rept/3-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.right/3","doc":"Returns the last characters in a text string.\nThis is Unicode safe.","ref":"Expression.V2.Callbacks.Standard.html#right/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.right/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ing\"`.\n\n```\n> right(\"testing\", 3)\n\"ing\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @right(\"testing\", 3) ...\"\n\"ing\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @right(\\\"testing\\\", 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"ing\", \" impact\"] = result\n    [\"chat for \", \"ing\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"right(\\\"testing\\\", 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"ing\" = result\n    \"ing\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@right(\\\"testing\\\", 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"ing\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#right/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.right/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"ту главы Госдепа США\"`.\n\n```\n> right(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20)\n\"ту главы Госдепа США\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @right(\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\", 20) ...\"\n\"ту главы Госдепа США\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @right(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"ту главы Госдепа США\", \" impact\"] = result\n    [\"chat for \", \"ту главы Госдепа США\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"right(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"ту главы Госдепа США\" = result\n    \"ту главы Госдепа США\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@right(\\\"Умерла Мадлен Олбрайт - первая женщина на посту главы Госдепа США\\\", 20)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"ту главы Госдепа США\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#right/3-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.second/2","doc":"Returns only the second of a datetime (0 to 59)","ref":"Expression.V2.Callbacks.Standard.html#second/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.second/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `57` when used with the following context:\n\n```elixir\n%{\"now\" => ~U[2024-01-16 12:53:57.311417Z]}\n```\n\n\n```\n> second(now)\n57\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @second(now) ...\"\n\"57\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @second(now) impact\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.311417Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 57, \" impact\"] = result\n    [\"chat for \", 57, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"second(now)\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.311417Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 57 = result\n    57\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@second(now)\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.311417Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"57\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#second/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.substitute/4","doc":"Substitutes new_text for old_text in a text string. If instance_num is given, then only that instance will be substituted","ref":"Expression.V2.Callbacks.Standard.html#substitute/4"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.substitute/4","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"I can do\"`.\n\n```\n> substitute(\"I can't\", \"can't\", \"can do\")\n\"I can do\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @substitute(\"I can't\", \"can't\", \"can do\") ...\"\n\"I can do\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @substitute(\\\"I can't\\\", \\\"can't\\\", \\\"can do\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"I can do\", \" impact\"] = result\n    [\"chat for \", \"I can do\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"substitute(\\\"I can't\\\", \\\"can't\\\", \\\"can do\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"I can do\" = result\n    \"I can do\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@substitute(\\\"I can't\\\", \\\"can't\\\", \\\"can do\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"I can do\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#substitute/4-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.sum_vargs/2","doc":"Returns the sum of all arguments, equivalent to the + operator\n\n```\nYou have @SUM(contact.reports, contact.forms) reports and forms\n```","ref":"Expression.V2.Callbacks.Standard.html#sum_vargs/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.sum_vargs/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `6`.\n\n```\n> sum(1, 2, 3)\n6\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @sum(1, 2, 3) ...\"\n\"6\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @sum(1, 2, 3) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 6, \" impact\"] = result\n    [\"chat for \", 6, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"sum(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 6 = result\n    6\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@sum(1, 2, 3)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"6\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#sum_vargs/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.time/4","doc":"Defines a time value which can be used for time arithmetic","ref":"Expression.V2.Callbacks.Standard.html#time/4"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.time/4","doc":"When used in the following Stack expression it returns a  value of type **Time**: `~T[12:13:14]`.\n\n```\n> time(12, 13, 14)\n~T[12:13:14]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @time(12, 13, 14) ...\"\n\"~T[12:13:14]\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @time(12, 13, 14) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~T[12:13:14], \" impact\"] = result\n    [\"chat for \", ~T[12:13:14], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"time(12, 13, 14)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~T[12:13:14] = result\n    ~T[12:13:14]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@time(12, 13, 14)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"~T[12:13:14]\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#time/4-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.timevalue/2","doc":"Converts time stored in text to an actual time","ref":"Expression.V2.Callbacks.Standard.html#timevalue/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.timevalue/2","doc":"When used in the following Stack expression it returns a  value of type **Time**: `~T[02:30:00]`.\n\n```\n> timevalue(\"2:30\")\n~T[02:30:00]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @timevalue(\"2:30\") ...\"\n\"~T[02:30:00]\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @timevalue(\\\"2:30\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~T[02:30:00], \" impact\"] = result\n    [\"chat for \", ~T[02:30:00], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"timevalue(\\\"2:30\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~T[02:30:00] = result\n    ~T[02:30:00]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@timevalue(\\\"2:30\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"~T[02:30:00]\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#timevalue/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.timevalue/2","doc":"When used in the following Stack expression it returns a  value of type **Time**: `~T[02:30:55]`.\n\n```\n> timevalue(\"2:30:55\")\n~T[02:30:55]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @timevalue(\"2:30:55\") ...\"\n\"~T[02:30:55]\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @timevalue(\\\"2:30:55\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", ~T[02:30:55], \" impact\"] = result\n    [\"chat for \", ~T[02:30:55], \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"timevalue(\\\"2:30:55\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert ~T[02:30:55] = result\n    ~T[02:30:55]\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@timevalue(\\\"2:30:55\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"~T[02:30:55]\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#timevalue/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.today/1","doc":"Returns the current date","ref":"Expression.V2.Callbacks.Standard.html#today/1"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.today/1","doc":"When used in the following Stack expression it returns a  value of type **Date**: `~D[2024-01-16]`.\n\n```\n> today()\n~D[2024-01-16]\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @today() ...\"\n\"2024-01-16\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @today() impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", ~D[2024-01-16], \" impact\"] = result\n    [\"chat for \", ~D[2024-01-16], \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"today()\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert ~D[2024-01-16] = result\n    ~D[2024-01-16]\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@today()\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2024-01-16\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#today/1-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.unichar/2","doc":"Returns the unicode character specified by a number","ref":"Expression.V2.Callbacks.Standard.html#unichar/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.unichar/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"A\"`.\n\n```\n> unichar(65)\n\"A\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unichar(65) ...\"\n\"A\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @unichar(65) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"A\", \" impact\"] = result\n    [\"chat for \", \"A\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"unichar(65)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"A\" = result\n    \"A\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@unichar(65)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"A\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#unichar/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.unichar/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"é\"`.\n\n```\n> unichar(233)\n\"é\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unichar(233) ...\"\n\"é\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @unichar(233) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"é\", \" impact\"] = result\n    [\"chat for \", \"é\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"unichar(233)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"é\" = result\n    \"é\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@unichar(233)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"é\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#unichar/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.unicode/2","doc":"Returns a numeric code for the first character in a text string","ref":"Expression.V2.Callbacks.Standard.html#unicode/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.unicode/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `65`.\n\n```\n> unicode(\"A\")\n65\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unicode(\"A\") ...\"\n\"65\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @unicode(\\\"A\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 65, \" impact\"] = result\n    [\"chat for \", 65, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"unicode(\\\"A\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 65 = result\n    65\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@unicode(\\\"A\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"65\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#unicode/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.unicode/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `233`.\n\n```\n> unicode(\"é\")\n233\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @unicode(\"é\") ...\"\n\"233\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @unicode(\\\"é\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 233, \" impact\"] = result\n    [\"chat for \", 233, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"unicode(\\\"é\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 233 = result\n    233\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@unicode(\\\"é\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"233\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#unicode/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.upper/2","doc":"Converts a text string to uppercase","ref":"Expression.V2.Callbacks.Standard.html#upper/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.upper/2","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"FOO\"`.\n\n```\n> upper(\"foo\")\n\"FOO\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @upper(\"foo\") ...\"\n\"FOO\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @upper(\\\"foo\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"FOO\", \" impact\"] = result\n    [\"chat for \", \"FOO\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"upper(\\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"FOO\" = result\n    \"FOO\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@upper(\\\"foo\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"FOO\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#upper/2-example-1"},{"type":"function","title":"Expression.V2.Callbacks.Standard.weekday/2","doc":"Returns the day of the week of a date (1 for Sunday to 7 for Saturday)","ref":"Expression.V2.Callbacks.Standard.html#weekday/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.weekday/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `1` when used with the following context:\n\n```elixir\n%{\"today\" => ~D[2022-11-06]}\n```\n\n\n```\n> weekday(today)\n1\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @weekday(today) ...\"\n\"1\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @weekday(today) impact\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-06]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 1, \" impact\"] = result\n    [\"chat for \", 1, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"weekday(today)\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-06]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 1 = result\n    1\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@weekday(today)\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-06]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"1\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#weekday/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.weekday/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3` when used with the following context:\n\n```elixir\n%{\"today\" => ~D[2022-11-01]}\n```\n\n\n```\n> weekday(today)\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @weekday(today) ...\"\n\"3\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @weekday(today) impact\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-01]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 3, \" impact\"] = result\n    [\"chat for \", 3, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"weekday(today)\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-01]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 3 = result\n    3\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@weekday(today)\",\n    ...>   Expression.V2.Context.new(%{\"today\" => ~D[2022-11-01]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#weekday/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word/3","doc":"Extracts the nth word from the given text string. If stop is a negative number,\nthen it is treated as count backwards from the end of the text. If by_spaces is\nspecified and is `true` then the function splits the text into words only by spaces.\nOtherwise the text is split by punctuation characters as well","ref":"Expression.V2.Callbacks.Standard.html#word/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"cow\"`.\n\n```\n> word(\"hello cow-boy\", 2)\n\"cow\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", 2) ...\"\n\"cow\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word(\\\"hello cow-boy\\\", 2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"cow\", \" impact\"] = result\n    [\"chat for \", \"cow\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word(\\\"hello cow-boy\\\", 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"cow\" = result\n    \"cow\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word(\\\"hello cow-boy\\\", 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"cow\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"cow-boy\"`.\n\n```\n> word(\"hello cow-boy\", 2, true)\n\"cow-boy\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", 2, true) ...\"\n\"cow-boy\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word(\\\"hello cow-boy\\\", 2, true) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"cow-boy\", \" impact\"] = result\n    [\"chat for \", \"cow-boy\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word(\\\"hello cow-boy\\\", 2, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"cow-boy\" = result\n    \"cow-boy\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word(\\\"hello cow-boy\\\", 2, true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"cow-boy\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.word/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"boy\"`.\n\n```\n> word(\"hello cow-boy\", -1)\n\"boy\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word(\"hello cow-boy\", -1) ...\"\n\"boy\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word(\\\"hello cow-boy\\\", -1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"boy\", \" impact\"] = result\n    [\"chat for \", \"boy\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word(\\\"hello cow-boy\\\", -1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"boy\" = result\n    \"boy\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word(\\\"hello cow-boy\\\", -1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"boy\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word/3-example-3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word/4","doc":"","ref":"Expression.V2.Callbacks.Standard.html#word/4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word_count/2","doc":"Returns the number of words in the given text string. If by_spaces is specified and is `true` then the function splits the text into words only by spaces. Otherwise the text is split by punctuation characters as well\n\n```\n> You entered @word_count(\"one two three\") words\nYou entered 3 words\n```","ref":"Expression.V2.Callbacks.Standard.html#word_count/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.word_count/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `3`.\n\n```\n> word_count(\"hello cow-boy\")\n3\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_count(\"hello cow-boy\") ...\"\n\"3\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_count(\\\"hello cow-boy\\\") impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 3, \" impact\"] = result\n    [\"chat for \", 3, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_count(\\\"hello cow-boy\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 3 = result\n    3\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_count(\\\"hello cow-boy\\\")\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"3\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_count/2-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.word_count/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `2`.\n\n```\n> word_count(\"hello cow-boy\", true)\n2\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_count(\"hello cow-boy\", true) ...\"\n\"2\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_count(\\\"hello cow-boy\\\", true) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", 2, \" impact\"] = result\n    [\"chat for \", 2, \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_count(\\\"hello cow-boy\\\", true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert 2 = result\n    2\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_count(\\\"hello cow-boy\\\", true)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_count/2-example-2"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word_count/3","doc":"","ref":"Expression.V2.Callbacks.Standard.html#word_count/3"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word_slice/3","doc":"Extracts a substring of the words beginning at start, and up to but not-including stop.\nIf stop is omitted then the substring will be all words from start until the end of the text.\nIf stop is a negative number, then it is treated as count backwards from the end of the text.\nIf by_spaces is specified and is `true` then the function splits the text into words only by spaces.\nOtherwise the text is split by punctuation characters as well","ref":"Expression.V2.Callbacks.Standard.html#word_slice/3"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"expressions are\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 2, 4)\n\"expressions are\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 2, 4) ...\"\n\"expressions are\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_slice(\\\"FLOIP expressions are fun\\\", 2, 4) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"expressions are\", \" impact\"] = result\n    [\"chat for \", \"expressions are\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 2, 4)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"expressions are\" = result\n    \"expressions are\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 2, 4)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"expressions are\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_slice/3-example-1"},{"type":"function","title":"Example 2: - Expression.V2.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"expressions are fun\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 2)\n\"expressions are fun\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 2) ...\"\n\"expressions are fun\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_slice(\\\"FLOIP expressions are fun\\\", 2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"expressions are fun\", \" impact\"] = result\n    [\"chat for \", \"expressions are fun\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"expressions are fun\" = result\n    \"expressions are fun\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"expressions are fun\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_slice/3-example-2"},{"type":"function","title":"Example 3: - Expression.V2.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"FLOIP expressions\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", 1, -2)\n\"FLOIP expressions\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", 1, -2) ...\"\n\"FLOIP expressions\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_slice(\\\"FLOIP expressions are fun\\\", 1, -2) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"FLOIP expressions\", \" impact\"] = result\n    [\"chat for \", \"FLOIP expressions\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", 1, -2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"FLOIP expressions\" = result\n    \"FLOIP expressions\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", 1, -2)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"FLOIP expressions\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_slice/3-example-3"},{"type":"function","title":"Example 4: - Expression.V2.Callbacks.Standard.word_slice/3","doc":"When used in the following Stack expression it returns a  value of type **String**: `\"fun\"`.\n\n```\n> word_slice(\"FLOIP expressions are fun\", -1)\n\"fun\"\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @word_slice(\"FLOIP expressions are fun\", -1) ...\"\n\"fun\"\n```\n\n    iex> # Evaluate a string with expressions\n    iex> import ExUnit.Assertions\n    iex> result = Expression.V2.eval(\n    ...>   \"chat for @word_slice(\\\"FLOIP expressions are fun\\\", -1) impact\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex> assert [\"chat for \", \"fun\", \" impact\"] = result\n    [\"chat for \", \"fun\", \" impact\"]\n    iex>\n    iex> # Evaluate a standalone expression block\n    iex> result = Expression.V2.eval_block(\n    ...>   \"word_slice(\\\"FLOIP expressions are fun\\\", -1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    iex>\n    iex> assert \"fun\" = result\n    \"fun\"\n    iex>\n    iex> # Evaluate a string with expressions into a single string\n    iex> Expression.V2.eval_as_string(\n    ...>   \"@word_slice(\\\"FLOIP expressions are fun\\\", -1)\",\n    ...>   Expression.V2.Context.new(%{}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"fun\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#word_slice/3-example-4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word_slice/4","doc":"","ref":"Expression.V2.Callbacks.Standard.html#word_slice/4"},{"type":"function","title":"Expression.V2.Callbacks.Standard.word_slice/5","doc":"","ref":"Expression.V2.Callbacks.Standard.html#word_slice/5"},{"type":"function","title":"Expression.V2.Callbacks.Standard.year/2","doc":"Returns only the year of a date","ref":"Expression.V2.Callbacks.Standard.html#year/2"},{"type":"function","title":"Example 1: - Expression.V2.Callbacks.Standard.year/2","doc":"When used in the following Stack expression it returns a  value of type **Integer**: `2024` when used with the following context:\n\n```elixir\n%{\"now\" => ~U[2024-01-16 12:53:57.312748Z]}\n```\n\n\n```\n> year(now)\n2024\n```\n\nWhen used as an expression in text, prepend it with an `@`:\n\n```expression\n> \"... @year(now) ...\"\n\"2024\"\n```\n\n    ..$> # Evaluate a string with expressions\n    ..$> import ExUnit.Assertions\n    ..$> result = Expression.V2.eval(\n    ...>   \"chat for @year(now) impact\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.312748Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$> assert [\"chat for \", 2024, \" impact\"] = result\n    [\"chat for \", 2024, \" impact\"]\n    ..$>\n    ..$> # Evaluate a standalone expression block\n    ..$> result = Expression.V2.eval_block(\n    ...>   \"year(now)\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.312748Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    ..$>\n    ..$> assert 2024 = result\n    2024\n    ..$>\n    ..$> # Evaluate a string with expressions into a single string\n    ..$> Expression.V2.eval_as_string(\n    ...>   \"@year(now)\",\n    ...>   Expression.V2.Context.new(%{\"now\" => ~U[2024-01-16 12:53:57.312748Z]}, Expression.V2.Callbacks.Standard)\n    ...> )\n    \"2024\"\n\n\n---","ref":"Expression.V2.Callbacks.Standard.html#year/2-example-1"},{"type":"module","title":"Expression.V2.Compat","doc":"Compatibility module to make the transition from V1 to V2 a bit easier, hopefully.\n\nIt does a few things:\n\n* It swaps out V2 callbacks for V1 callbacks when evaluating expressions with V1.\n* It does some patching of the context to match V1's assumptions:\n    * case insensitive context keys\n    * casting of integers\n    * casting of datetimes\n* It compares the output of V1 to V2, if those aren't equal it will log an error and return the V1 response.\n* If there is no error it will return the value from V2.\n\n> **NOTE**: This module does *twice* the work because it runs V1 and V2 sequentially\n  and then compares the result before returning a value.\n\n> **NOTE**: This was throwing more errors in prod than anticipated, hacking in a revert temporarily","ref":"Expression.V2.Compat.html"},{"type":"function","title":"Expression.V2.Compat.attempt_boolean/1","doc":"","ref":"Expression.V2.Compat.html#attempt_boolean/1"},{"type":"function","title":"Expression.V2.Compat.attempt_datetime/1","doc":"","ref":"Expression.V2.Compat.html#attempt_datetime/1"},{"type":"function","title":"Expression.V2.Compat.attempt_float/1","doc":"","ref":"Expression.V2.Compat.html#attempt_float/1"},{"type":"function","title":"Expression.V2.Compat.attempt_integer/1","doc":"","ref":"Expression.V2.Compat.html#attempt_integer/1"},{"type":"function","title":"Expression.V2.Compat.evaluate!/3","doc":"","ref":"Expression.V2.Compat.html#evaluate!/3"},{"type":"function","title":"Expression.V2.Compat.evaluate_as_string!/3","doc":"","ref":"Expression.V2.Compat.html#evaluate_as_string!/3"},{"type":"function","title":"Expression.V2.Compat.evaluate_block!/3","doc":"","ref":"Expression.V2.Compat.html#evaluate_block!/3"},{"type":"function","title":"Expression.V2.Compat.log_error/4","doc":"","ref":"Expression.V2.Compat.html#log_error/4"},{"type":"function","title":"Expression.V2.Compat.normalize_value/1","doc":"","ref":"Expression.V2.Compat.html#normalize_value/1"},{"type":"function","title":"Expression.V2.Compat.patch_v1_context/1","doc":"","ref":"Expression.V2.Compat.html#patch_v1_context/1"},{"type":"function","title":"Expression.V2.Compat.patch_v1_key/1","doc":"","ref":"Expression.V2.Compat.html#patch_v1_key/1"},{"type":"function","title":"Expression.V2.Compat.return_or_raise/4","doc":"","ref":"Expression.V2.Compat.html#return_or_raise/4"},{"type":"function","title":"Expression.V2.Compat.return_or_raise_binaries/4","doc":"","ref":"Expression.V2.Compat.html#return_or_raise_binaries/4"},{"type":"function","title":"Expression.V2.Compat.unpack_returned_value/1","doc":"","ref":"Expression.V2.Compat.html#unpack_returned_value/1"},{"type":"module","title":"Expression.V2.Compile","doc":"An compiler for AST returned by Expression.V2.Parser.\n\nThis reads the AST output returned by `Expression.V2.parse/1` and\ncompiles it to Elixir code.\n\nIt does this by emitting valid Elixir AST, mimicking what `quote/2` does.\n\nThe Elixir AST is then supplied to `Code.eval_quoted_with_env/3` without any\nvariable binding. What is returned is an anonymous function that accepts an\n`Expression.V2.Context.t` struct and evaluates the code against that context.\n\nAny function calls are applied to the callback module referenced in the context.\nSo if an expression uses a function called `foo(1, 2, 3)` then the callback's\n`callback/3` function will be called as follows:\n\n```elixir\napply(context.callback_module, :callback, [\"foo\", [1, 2, 3]])\n```\n\nThere is some special handling of some functions that have specific Elixir AST\nsyntax requirements.\n\nThese are documented in the `to_quoted/2` function.\n\nAll variables referenced by the expression are scoped to `context.vars`.\nHowever the full context is supplied to any function calls, giving\nfunctions the privilege of doing more than the `context.vars` scope alone\nwould allow them to do.","ref":"Expression.V2.Compile.html"},{"type":"function","title":"Expression.V2.Compile.compile/1","doc":"Accepts AST as emitted by `Expression.V2.parse/1` and returns an anonymous function\nthat accepts a Context.t as an argument and returns the result  of the expression\nagainst the given Context.\n\nIf the callback functions defined in the callback module are pure then this function\nis also pure and is suitable for caching.","ref":"Expression.V2.Compile.html#compile/1"},{"type":"function","title":"Expression.V2.Compile.to_quoted/1","doc":"Convert the AST returned from `Expression.V2.parse/1` into valid Elixir AST\nthat can be used by `Code.eval_quoted_with_env/3`.\n\nThere is some special handling here:\n\n1. Lists are recursed to ensure that all list items are properly quoted.\n2. \"\"Quoted strings\"\" are unquoted and returned as regular strings to the AST.\n3. \"Normal strings\" are converted into Atoms and treated as such during eval.\n4. Literals such as numbers & booleans are left as is.\n5. Range.t items are converted to valid Elixir AST.\n6. `&` and `&1` captures are generated into valid Elixir AST captures.\n7. Any functions are generated as being function calls for the given callback module.","ref":"Expression.V2.Compile.html#to_quoted/1"},{"type":"function","title":"Expression.V2.Compile.wrap_in_context/1","doc":"Wrap an AST block into an anonymous function that accepts\na single argument called context.\n\nThis happens _after_ all the code generation completes. The code\ngenerated expects a variable called `context` to exist, wrapping\nit in this function ensures that it does.\n\nThis is the anonymous function that is returned to the caller.\nThe caller is then responsible to call it with the correct context\nvariables.","ref":"Expression.V2.Compile.html#wrap_in_context/1"},{"type":"module","title":"Expression.V2.Context","doc":"The context supplied to a function generated by `Expression.V2.Compile.compile/1`\n\nThis will be expanded with support for more attributes that a callback function\ncan access but normal Expression evaluation can not.","ref":"Expression.V2.Context.html"},{"type":"function","title":"Expression.V2.Context.new/2","doc":"","ref":"Expression.V2.Context.html#new/2"},{"type":"function","title":"Expression.V2.Context.private/3","doc":"","ref":"Expression.V2.Context.html#private/3"},{"type":"type","title":"Expression.V2.Context.t/0","doc":"","ref":"Expression.V2.Context.html#t:t/0"},{"type":"module","title":"Expression.V2.Parser","doc":"A NimbleParsec parser for FLOIP expressions.\n\nFLOIP Expressions consist of plain text and of blocks. Plain text is returned untouched\nbut blocks are evaluated.\n\nBlocks are prefixed with an `@` sign. Blocks can either have expressions between brackets or\nbe used in a shorthand form when wanting to use a single function or variable substitution.\n\nAs an example, the following are identical:\n\n* `@(now())` and `@now()`\n* `@contact.name` and `@(contact.name)`\n\nHowever, a full expression needs to be within brackets:\n\n`Tomorrow's is @(today().day + 1)`\n\nThis parses it into an Abstract Syntax Tree (AST) which follows a style much like a Lisp would.\nIt parses expressions in [Infix notation](https://en.wikipedia.org/wiki/Infix_notation) such as\n`1 + 1` and parses it into lists where the operator is the first element and the second element\nis the list of arguments for the operator.\n\n```\n[\"+\", [1, 1]]\n```\n\nFunctions are expressed as:\n\n```\n{\"function name\", [arg1, arg2]}\n```\n\nUntil we have a fixed scope of allowed functions, or if we can dynamically look up whether an\n`atom` is a function or a variable reference, we will need to rely on tuples to represent functions\nas otherwise the system has no means to distinguish the following AST as being a function call\nor a list as a variable:\n\n```\n[\"echo\", [1, 2, 3]]\n```\n\nWithout being able to say _ahead_ of time whether or not `echo/1` is a known function, the\nsystem cannot reliable determine whether the result of this AST should be `[\"echo\", [1, 2, 3]]`\nor the result of `echo(1, 2, 3)`.\n\nVariable references are single values.\n\n```\n\"contact\"\n```\n\nThis module provides two functions for parsing. `parse/2` which will parse a full FLOIP expression\nincluding text and blocks, and `expression/2` which will parse expression blocks.\n\nInternally `parse/2` refers to the same parsers as `expression/2` for things that are expressions.","ref":"Expression.V2.Parser.html"},{"type":"function","title":"Expression.V2.Parser.expression/2","doc":"Parse a block and return the AST","ref":"Expression.V2.Parser.html#expression/2"},{"type":"function","title":"Example - Expression.V2.Parser.expression/2","doc":"iex> Expression.V2.Parser.expression(\"contact.age + 1\")\n    {:ok,  [{\"+\", [{\"__property__\", [\"contact\", \"age\"]}, 1]}], \"\", %{}, {1, 0}, 15}","ref":"Expression.V2.Parser.html#expression/2-example"},{"type":"function","title":"Expression.V2.Parser.parse/2","doc":"Parse an expression and return the AST","ref":"Expression.V2.Parser.html#parse/2"},{"type":"function","title":"Example - Expression.V2.Parser.parse/2","doc":"iex> Expression.V2.Parser.parse(\"hello @world the time is @now()\")\n    {:ok, [\"hello \", [\"world\"], \" the time is \", [{\"now\", []}]], \"\", %{}, {1, 0}, 31}","ref":"Expression.V2.Parser.html#parse/2-example"},{"type":"function","title":"Expression.V2.Parser.to_double_quoted_string/1","doc":"","ref":"Expression.V2.Parser.html#to_double_quoted_string/1"}],"content_type":"text/markdown"}